@@ -77,6 +77,7 @@ qint64 QFSFileEngine::size() const
     Q_D(const QFSFileEngine);
     QT_STATBUF st;
     int ret = 0;
+    const_cast<QFSFileEngine *>(this)->flush();
     if(d->fd != -1)
         ret = QT_FSTAT(d->fd, &st);
     else
@@ -182,7 +183,8 @@ QStringList QFSFileEngine::entryList(QDir::Filters filters, const QStringList &f
 #endif // _POSIX_THREAD_SAFE_FUNCTIONS
     {
         QString fn = QFile::decodeName(QByteArray(file->d_name));
-        fi.setFile(d->file + QLatin1Char('/') + fn);
+        const QString filePath = d->file + QLatin1Char('/') + fn;
+        fi.setFile(filePath);
 
 #ifndef QT_NO_REGEXP
         if(!((filters & QDir::AllDirs) && fi.isDir())) {
@@ -201,7 +203,12 @@ QStringList QFSFileEngine::entryList(QDir::Filters filters, const QStringList &f
 #endif
         if ((filters & QDir::NoDotAndDotDot) && ((fn == QLatin1String(".") || fn == QLatin1String(".."))))
             continue;
-        bool isHidden = (fn.at(0) == QLatin1Char('.') && fn.length() > 1 && fn != QLatin1String(".."));
+        bool isHidden = (fn.at(0) == QLatin1Char('.') && fn.length() > 1 && fn != QLatin1String("..")
+#if defined(Q_WS_MAC)
+            || d->isMacHidden(filePath)
+#endif
+        );
+
         if (!includeHidden && isHidden)
             continue;
 
@@ -234,7 +241,7 @@ QStringList QFSFileEngine::entryList(QDir::Filters filters, const QStringList &f
         ret.append(fn);
     }
     if (closedir(dir) != 0) {
-        qWarning("QDir::readDirEntries: Cannot close the directory: %s", d->file.toLocal8Bit().data());
+        qWarning("QDir::readDirEntries: Cannot close directory: %s", d->file.toLocal8Bit().data());
     }
     return ret;
 }
@@ -324,7 +331,30 @@ bool QFSFileEnginePrivate::isSymlink() const
     return is_link;
 }
 
-QAbstractFileEngine::FileFlags QFSFileEngine::fileFlags(QAbstractFileEngine::FileFlags type) const
+#if defined (Q_WS_MAC)
+bool QFSFileEnginePrivate::isMacHidden(const QString &path) const
+{
+    OSErr err = noErr;
+    FSRef fsRef;
+    err = FSPathMakeRef((const UInt8 *)QFile::encodeName(QDir::cleanPath(path)).data(), &fsRef, 0);
+    if (err != noErr)
+        return false;
+
+    FSCatalogInfo catInfo;
+    err = FSGetCatalogInfo(&fsRef, kFSCatInfoFinderInfo, &catInfo, NULL, NULL, NULL);
+    if (err != noErr)
+        return false;
+
+    FileInfo * const fileInfo = reinterpret_cast<FileInfo*>(&catInfo.finderInfo);
+    bool result = (fileInfo->finderFlags & kIsInvisible);
+    return result;
+}
+#endif
+
+/*!
+    \reimp
+*/
+QAbstractFileEngine::FileFlags QFSFileEngine::fileFlags(FileFlags type) const
 {
     Q_D(const QFSFileEngine);
     // Force a stat, so that we're guaranteed to get up-to-date results
@@ -369,7 +399,8 @@ QAbstractFileEngine::FileFlags QFSFileEngine::fileFlags(QAbstractFileEngine::Fil
         bool foundAlias = false;
         {
             FSRef fref;
-            if(FSPathMakeRef((const UInt8 *)QFile::encodeName(d->file).data(), &fref, NULL) == noErr) {
+            if(FSPathMakeRef((const UInt8 *)QFile::encodeName(QDir::cleanPath(d->file)).data(),
+                             &fref, NULL) == noErr) {
                 Boolean isAlias, isFolder;
                 if(FSIsAliasFile(&fref, &isAlias, &isFolder) == noErr && isAlias) {
                     foundAlias = true;
@@ -392,7 +423,11 @@ QAbstractFileEngine::FileFlags QFSFileEngine::fileFlags(QAbstractFileEngine::Fil
         ret |= LocalDiskFlag;
         if (exists)
             ret |= ExistsFlag;
-        if(fileName(BaseName)[0] == QLatin1Char('.'))
+        if(fileName(BaseName)[0] == QLatin1Char('.')
+#if defined(Q_WS_MAC)
+            || d->isMacHidden(d->file)
+#endif
+        )
             ret |= HiddenFlag;
         if(d->file == QLatin1String("/"))
             ret |= RootFlag;
@@ -483,7 +518,8 @@ QString QFSFileEngine::fileName(FileName file) const
 
                 if (!ret.startsWith(QLatin1Char('/'))) {
                     if (d->file.startsWith(QLatin1Char('/'))) {
-                        ret.prepend(d->file.left(d->file.lastIndexOf('/')) + QLatin1Char('/'));
+                        ret.prepend(d->file.left(d->file.lastIndexOf(QLatin1Char('/')))
+                                    + QLatin1Char('/'));
                     } else {
                         ret.prepend(QDir::currentPath() + QLatin1Char('/'));
                     }
@@ -497,7 +533,7 @@ QString QFSFileEngine::fileName(FileName file) const
 #if !defined(QWS) && defined(Q_OS_MAC)
         {
             FSRef fref;
-            if(FSPathMakeRef((const UInt8 *)QFile::encodeName(d->file).data(), &fref, NULL) == noErr) {
+            if(FSPathMakeRef((const UInt8 *)QFile::encodeName(QDir::cleanPath(d->file)).data(), &fref, 0) == noErr) {
                 Boolean isAlias, isFolder;
                 if(FSResolveAliasFile(&fref, true, &isFolder, &isAlias) == noErr && isAlias) {
                     AliasHandle alias;
