@@ -39,6 +39,7 @@
 #include "QtCore/qbytearray.h"
 #include "QtCore/qobjectdefs.h"
 #include "QtCore/qstring.h"
+#include "private/qringbuffer_p.h"
 #ifndef QT_NO_QOBJECT
 #include "private/qobject_p.h"
 #endif
@@ -57,7 +58,24 @@ public:
     QIODevice::OpenMode openMode;
     QString errorString;
 
-    QByteArray ungetBuffer;
+    QRingBuffer buffer;
+    qint64 pos;
+    qint64 devicePos;
+    bool baseReadLineDataCalled;
+
+    enum AccessMode {
+        Unset,
+        Sequential,
+        RandomAccess
+    };
+    mutable AccessMode accessMode;
+    inline bool isSequential() const
+    {
+        if (accessMode == Unset)
+            accessMode = q_func()->isSequential() ? Sequential : RandomAccess;
+        return accessMode == Sequential;
+    }
+    
 
 #ifdef QT_NO_QOBJECT
     QIODevice *q_ptr;
