@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -31,9 +31,184 @@
 #include <stdlib.h>
 #include <errno.h>
 
-#ifndef Q_OS_WIN
-# define HAS_MKSTEMP
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <stdlib.h>
+#include <time.h>
+#include <ctype.h>
+#include <sys/types.h>
+
+#ifdef Q_OS_WIN
+#include <process.h>
+#if defined(_MSC_VER) && _MSC_VER >= 1400
+#include <share.h>
 #endif
+#endif
+
+/*
+ * Copyright (c) 1987, 1993
+ *	The Regents of the University of California.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the University nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ */
+static int _gettemp(char *path, int *doopen, int domkdir, int slen)
+{
+	char *start, *trv, *suffp;
+	struct stat sbuf;
+	int rval;
+#ifdef Q_OS_WIN
+    int pid;
+#else
+	pid_t pid;
+#endif
+
+	if (doopen && domkdir) {
+		errno = EINVAL;
+		return(0);
+	}
+
+	for (trv = path; *trv; ++trv)
+		;
+	trv -= slen;
+	suffp = trv;
+	--trv;
+	if (trv < path) {
+		errno = EINVAL;
+		return (0);
+	}
+#if defined(Q_OS_WIN) && defined(_MSC_VER) && _MSC_VER >= 1400
+        pid = _getpid();
+#else
+        pid = getpid();
+#endif
+	while (trv >= path && *trv == 'X' && pid != 0) {
+		*trv-- = (pid % 10) + '0';
+		pid /= 10;
+        }
+
+#ifndef S_ISDIR
+#define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
+#endif
+
+        while (trv >= path && *trv == 'X') {
+            char c;
+
+            // CHANGE arc4random() -> random()
+            pid = (qrand() & 0xffff) % (26+26);
+		if (pid < 26)
+			c = pid + 'A';
+		else
+			c = (pid - 26) + 'a';
+		*trv-- = c;
+	}
+	start = trv + 1;
+
+	/*
+	 * check the target directory; if you have six X's and it
+	 * doesn't exist this runs for a *very* long time.
+	 */
+	if (doopen || domkdir) {
+		for (;; --trv) {
+			if (trv <= path)
+				break;
+			if (*trv == '/') {
+				*trv = '\0';
+				rval = stat(path, &sbuf);
+				*trv = '/';
+				if (rval != 0)
+					return(0);
+				if (!S_ISDIR(sbuf.st_mode)) {
+					errno = ENOTDIR;
+					return(0);
+				}
+				break;
+			}
+		}
+	}
+
+	for (;;) {
+		if (doopen) {
+#if defined(Q_OS_WIN) && defined(_MSC_VER) && _MSC_VER >= 1400
+                        if (_sopen_s(doopen, path, O_CREAT|O_EXCL|O_RDWR|O_BINARY, _SH_DENYNO, _S_IREAD | _S_IWRITE)== 0)
+#else
+                        if ((*doopen =
+                            open(path, O_CREAT|O_EXCL|O_RDWR
+#  if defined(Q_OS_WIN)
+                                 |O_BINARY
+#  endif
+                                 , 0600)) >= 0)
+#endif
+
+				return(1);
+			if (errno != EEXIST)
+				return(0);
+		} else if (domkdir) {
+#ifdef Q_OS_WIN
+                    if (QT_MKDIR(path) == 0)
+#else
+                    if (mkdir(path, 0700) == 0)
+#endif
+				return(1);
+			if (errno != EEXIST)
+				return(0);
+            }
+#ifndef Q_OS_WIN
+            else if (lstat(path, &sbuf))
+			return(errno == ENOENT ? 1 : 0);
+#endif
+
+		/* tricky little algorwwithm for backward compatibility */
+		for (trv = start;;) {
+			if (!*trv)
+				return (0);
+			if (*trv == 'Z') {
+				if (trv == suffp)
+					return (0);
+				*trv++ = 'a';
+			} else {
+				if (isdigit(*trv))
+					*trv = 'a';
+				else if (*trv == 'z')	/* inc from z to A */
+					*trv = 'A';
+				else {
+					if (trv == suffp)
+						return (0);
+					++*trv;
+				}
+				break;
+			}
+		}
+	}
+	/*NOTREACHED*/
+}
+
+static int qt_mkstemps(char *path, int slen)
+{
+	int fd;
+	return (_gettemp(path, &fd, 0, slen) ? fd : -1);
+}
 
 //************* QTemporaryFileEngine
 class QTemporaryFileEngine : public QFSFileEngine
@@ -44,6 +219,8 @@ public:
     ~QTemporaryFileEngine();
 
     bool open(QIODevice::OpenMode flags);
+    bool remove();
+    bool close();
 };
 
 QTemporaryFileEngine::~QTemporaryFileEngine()
@@ -55,27 +232,15 @@ bool QTemporaryFileEngine::open(QIODevice::OpenMode)
     Q_D(QFSFileEngine);
 
     QString qfilename = d->file;
-    if(!qfilename.endsWith(QLatin1String("XXXXXX")))
+    if(!qfilename.contains(QLatin1String("XXXXXX")))
         qfilename += QLatin1String(".XXXXXX");
+
+    int suffixLength = qfilename.length() - (qfilename.lastIndexOf(QLatin1String("XXXXXX"), -1, Qt::CaseSensitive) + 6);
     d->closeFileHandle = true;
     char *filename = qstrdup(qfilename.toLocal8Bit());
 
-#ifdef HAS_MKSTEMP
-    d->fd = mkstemp(filename);
-#else
-#if defined(_MSC_VER) && _MSC_VER >= 1400
-    int len = int(strlen(filename)) + 1;
-    if(_mktemp_s(filename, len) == 0) {
-#else
-    if(mktemp(filename)) {
-#endif
-        int oflags = QT_OPEN_RDWR | QT_OPEN_CREAT;
-#if defined(Q_OS_MSDOS) || defined(Q_OS_WIN32) || defined(Q_OS_OS2)
-        oflags |= QT_OPEN_BINARY; // we handle all text translations our self.
-#endif
-        d->fd = d->sysOpen(filename, oflags);
-    }
-#endif
+    d->fd = qt_mkstemps(filename, suffixLength);
+
     if(d->fd != -1) {
         d->file = QString::fromLocal8Bit(filename); //changed now!
         delete [] filename;
@@ -87,6 +252,24 @@ bool QTemporaryFileEngine::open(QIODevice::OpenMode)
     return false;
 }
 
+bool QTemporaryFileEngine::remove()
+{
+    Q_D(QFSFileEngine);
+    // Since the QTemporaryFileEngine::close() does not really close the file,
+    // we must explicitly call QFSFileEngine::close() before we remove it.
+    QFSFileEngine::close();
+    bool removed = QFSFileEngine::remove();
+    d->file.clear();
+    return removed;
+}
+
+bool QTemporaryFileEngine::close()
+{
+    // Don't close the file, just seek to the front.
+    seek(0);
+    setError(QFile::UnspecifiedError, QString());
+    return true;
+}
 
 //************* QTemporaryFilePrivate
 class QTemporaryFilePrivate : public QFilePrivate
@@ -122,16 +305,36 @@ QTemporaryFilePrivate::~QTemporaryFilePrivate()
     \ingroup io
     \mainclass
 
-    QTemporaryFile is an I/O device that will get its input/output
-    from the local disk. The filename for the temporary file will be
-    guaranteed to be unique once the file is opened and will
+    QTemporaryFile is used to create unique temporary files safely.
+    The file itself is created by calling open(). The name of the
+    temporary file is guaranteed to be unique (i.e., you are
+    guaranteed to not overwrite an existing file), and the file will
     subsequently be removed upon destruction of the QTemporaryFile
-    object.
+    object. This is an important technique that avoids data
+    corruption for applications that store data in temporary files.
+    The file name is either auto-generated, or created based on a
+    template, which is passed to QTemporaryFile's constructor.
 
-    A temporary file will have some static part of the name and some
-    part that is calculated to be unique. The default filename qt_temp
-    will be placed into the temporary path as returned by
-    QDir::tempPath().
+    Example:
+
+    \code
+        {
+            QTemporaryFile file;
+            if (file.open()) {
+                // file.fileName() returns the unique file name
+            }
+
+            // the QTemporaryFile destructor removes the temporary file
+        }
+    \endcode
+
+    Reopening a QTemporaryFile after calling close() is safe. For as long as
+    the QTemporaryFile object itself is not destroyed, the unique temporary
+    file will exist and be kept open internally by QTemporaryFile.
+
+    A temporary file will have some static part of the name and some part that
+    is calculated to be unique. The default filename qt_temp will be placed
+    into the temporary path as returned by QDir::tempPath().
 
     \sa QDir::tempPath(), QFile
 */
@@ -141,7 +344,7 @@ QTemporaryFile::QTemporaryFile()
     : QFile(*new QTemporaryFilePrivate)
 {
     Q_D(QTemporaryFile);
-    d->templateName = QDir::tempPath() + "/" + QLatin1String("qt_temp.XXXXXX");
+    d->templateName = QDir::tempPath() + QLatin1String("/qt_temp.XXXXXX");
 }
 
 QTemporaryFile::QTemporaryFile(const QString &templateName)
@@ -153,9 +356,10 @@ QTemporaryFile::QTemporaryFile(const QString &templateName)
 
 #else
 /*!
-    Constructs a QTemporaryFile with no name.
+    Constructs a QTemporaryFile in QDir::tempPath(), using the file template
+    "qt_temp.XXXXXX". The file is stored in the system's temporary directory.
 
-    \sa setFileTemplate()
+    \sa setFileTemplate(), QDir::tempPath()
 */
 QTemporaryFile::QTemporaryFile()
     : QFile(*new QTemporaryFilePrivate, 0)
@@ -166,22 +370,25 @@ QTemporaryFile::QTemporaryFile()
 
 /*!
     Constructs a QTemporaryFile with a template filename of \a
-    templateName. Upon opening the temporary file this will be used to
-    create a unique filename. If the \a templateName does end in
-    XXXXXX it will automatically be appended and used as the dynamic
-    portion of the filename.
+    templateName. Upon opening the temporary file this will be used to create
+    a unique filename. If the \a templateName does not contain XXXXXX it will
+    automatically be appended and used as the dynamic portion of the filename.
+
+    If \a templateName is a relative path, the path will be relative to the
+    current working directory. You can use QDir::tempPath() to construct \a
+    templateName if you want use the system's temporary directory.
 
     \sa open(), fileTemplate()
 */
 QTemporaryFile::QTemporaryFile(const QString &templateName)
     : QFile(*new QTemporaryFilePrivate, 0)
 {
-    Q_D(QTemporaryFile);
-    d->templateName = templateName;
+    setFileTemplate(templateName);
 }
 
 /*!
-    Constructs a QTemporaryFile with the given \a parent, but with no name.
+    Constructs a QTemporaryFile (with the given \a parent) in
+    QDir::tempPath(), using the file template "qt_temp.XXXXXX".
 
     \sa setFileTemplate()
 */
@@ -189,7 +396,7 @@ QTemporaryFile::QTemporaryFile(QObject *parent)
     : QFile(*new QTemporaryFilePrivate, parent)
 {
     Q_D(QTemporaryFile);
-    d->templateName = QDir::tempPath() + QLatin1String("qt_temp.XXXXXX");
+    d->templateName = QDir::tempPath() + QLatin1String("/qt_temp.XXXXXX");
 }
 
 /*!
@@ -200,13 +407,16 @@ QTemporaryFile::QTemporaryFile(QObject *parent)
     XXXXXX it will automatically be appended and used as the dynamic
     portion of the filename.
 
+    If \a templateName is a relative path, the path will be relative to the
+    current working directory. You can use QDir::tempPath() to construct \a
+    templateName if you want use the system's temporary directory.
+
     \sa open(), fileTemplate()
 */
 QTemporaryFile::QTemporaryFile(const QString &templateName, QObject *parent)
     : QFile(*new QTemporaryFilePrivate, parent)
 {
-    Q_D(QTemporaryFile);
-    d->templateName = templateName;
+    setFileTemplate(templateName);
 }
 #endif
 
@@ -301,6 +511,10 @@ QString QTemporaryFile::fileTemplate() const
    the unique part of the filename, otherwise a filename will be
    determined automatically based on the static portion specified.
 
+    If \a name contains a relative file path, the path will be relative to the
+    current working directory. You can use QDir::tempPath() to construct \a
+    name if you want use the system's temporary directory.
+
    \sa fileTemplate()
 */
 void QTemporaryFile::setFileTemplate(const QString &name)
@@ -373,11 +587,19 @@ QAbstractFileEngine *QTemporaryFile::fileEngine() const
 
 /*!
    \reimp
-*/
 
+    Creates a unique file name for the temporary file, and opens it.  You can
+    get the unique name later by calling fileName(). The file is guaranteed to
+    have been created by this function (i.e., it has never existed before).
+*/
 bool QTemporaryFile::open(OpenMode flags)
 {
     Q_D(QTemporaryFile);
+    if (!d->fileName.isEmpty()) {
+        setOpenMode(flags);
+        return true;
+    }
+
     if (QFile::open(flags)) {
         d->fileName = d->fileEngine->fileName(QAbstractFileEngine::DefaultName);
         return true;
