@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -35,11 +35,11 @@
 // We mean it.
 //
 
-#if !defined (QT_QWS_NO_SHM)
-
 #include "qplatformdefs.h"
 #include "QtCore/qstring.h"
 
+#if !defined(QT_NO_QWS_MULTIPROCESS)
+
 class Q_CORE_EXPORT QSharedMemory {
 public:
 
@@ -78,6 +78,6 @@ private:
     key_t key;
 };
 
-#endif
+#endif // QT_NO_QWS_MULTIPROCESS
 
 #endif // QSHAREDMEMORY_P_H
