@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -26,6 +26,8 @@
 
 #include <QtGui/qdialog.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_COLORDIALOG
@@ -70,16 +72,18 @@ private:
 private:
     Q_DISABLE_COPY(QColorDialog)
 
-    Q_PRIVATE_SLOT(d_func(), void addCustom())
+    Q_PRIVATE_SLOT(d_func(), void _q_addCustom())
 
-    Q_PRIVATE_SLOT(d_func(), void newHsv(int h, int s, int v))
-    Q_PRIVATE_SLOT(d_func(), void newColorTypedIn(QRgb rgb))
-    Q_PRIVATE_SLOT(d_func(), void newCustom(int, int))
-    Q_PRIVATE_SLOT(d_func(), void newStandard(int, int))
+    Q_PRIVATE_SLOT(d_func(), void _q_newHsv(int h, int s, int v))
+    Q_PRIVATE_SLOT(d_func(), void _q_newColorTypedIn(QRgb rgb))
+    Q_PRIVATE_SLOT(d_func(), void _q_newCustom(int, int))
+    Q_PRIVATE_SLOT(d_func(), void _q_newStandard(int, int))
 
     friend class QColorShower;
 };
 
 #endif // QT_NO_COLORDIALOG
 
+QT_END_HEADER
+
 #endif // QCOLORDIALOG_H
