@@ -88,22 +88,22 @@ private:
 
 /*
   Encapsulation of conversion between MIME and Mac flavor.
-  Not need on X11, as the underlying protocol uses the MIME standard
+  Not needed on X11, as the underlying protocol uses the MIME standard
   directly.
 */
 
-class Q_GUI_EXPORT QMacMime {
+class Q_GUI_EXPORT QMacMime { //Obsolete
     char type;
 public:
     enum QMacMimeType { MIME_DND=0x01, MIME_CLIP=0x02, MIME_QT_CONVERTOR=0x04, MIME_ALL=MIME_DND|MIME_CLIP };
-    explicit QMacMime(char);
-    virtual ~QMacMime();
+    explicit QMacMime(char) { }
+    virtual ~QMacMime() { }
 
-    static void initialize();
+    static void initialize() { }
 
-    static QList<QMacMime*> all(QMacMimeType);
-    static QMacMime *convertor(QMacMimeType, const QString &mime, int flav);
-    static QString flavorToMime(QMacMimeType, int flav);
+    static QList<QMacMime*> all(QMacMimeType) { return QList<QMacMime*>(); }
+    static QMacMime *convertor(QMacMimeType, const QString &, int) { return 0; }
+    static QString flavorToMime(QMacMimeType, int) { return QString(); }
 
     virtual QString convertorName()=0;
     virtual int countFlavors()=0;
@@ -115,6 +115,33 @@ public:
     virtual QList<QByteArray> convertFromMime(const QString &mime, QVariant data, int flav)=0;
 };
 
+class Q_GUI_EXPORT QMacPasteboardMime {
+    char type;
+public:
+    enum QMacPasteboardMimeType { MIME_DND=0x01,
+                                  MIME_CLIP=0x02,
+                                  MIME_QT_CONVERTOR=0x04,
+                                  MIME_QT3_CONVERTOR=0x08,
+                                  MIME_ALL=MIME_DND|MIME_CLIP
+    };
+    explicit QMacPasteboardMime(char);
+    virtual ~QMacPasteboardMime();
+
+    static void initialize();
+
+    static QList<QMacPasteboardMime*> all(uchar);
+    static QMacPasteboardMime *convertor(uchar, const QString &mime, QString flav);
+    static QString flavorToMime(uchar, QString flav);
+
+    virtual QString convertorName() = 0;
+
+    virtual bool canConvert(const QString &mime, QString flav) = 0;
+    virtual QString mimeFor(QString flav) = 0;
+    virtual QString flavorFor(const QString &mime) = 0;
+    virtual QVariant convertToMime(const QString &mime, QList<QByteArray> data, QString flav) = 0;
+    virtual QList<QByteArray> convertFromMime(const QString &mime, QVariant data, QString flav) = 0;
+};
+
 #endif // Q_WS_MAC
 
 QT_END_HEADER
