@@ -26,32 +26,15 @@
 
 #if (defined(QT_HAVE_SSE) && (!defined(__APPLE__) || defined(__i386__))) || defined(QT_HAVE_IWMMXT)
 
-#if defined(Q_CC_GNU) && !defined(Q_CC_INTEL)
-#  include <mmintrin.h>
-#  if !defined(__IWMMXT__)
-#    include <xmmintrin.h>
-#  endif
-// It seems that gcc (3.1 <= x < 3.4) segfaults when casting the ULL immediate values to __m64.
-// A workaround was proposed here: http://gcc.gnu.org/ml/gcc-prs/2002-07/msg00329.html
-#  if !defined(Q_CC_INTEL) && __GNUC__ == 3 && __GNUC_MINOR__ >= 1 && __GNUC_MINOR__ < 4
-#    define C_FF volatile unsigned long long mmx_0x00ff_ull = 0x00ff00ff00ff00ffULL; \
-		 const m64 mmx_0x00ff = (__m64)mmx_0x00ff_ull
-#    define C_80 volatile unsigned long long mmx_0x0080_ull = 0x0080008000800080ULL; \
-                 const m64 mmx_0x0080 = (__m64)mmx_0x0080_ull
-#  else
-#    define C_FF const m64 mmx_0x00ff = (__m64)0x00ff00ff00ff00ffULL
-#    define C_80 const m64 mmx_0x0080 = (__m64)0x0080008000800080ULL
-#  endif
-#  define C_00 const m64 mmx_0x0000 = _mm_setzero_si64()
-#elif defined(Q_CC_INTEL) || defined(Q_OS_WIN)
-#  include <mmintrin.h>
+#include <mmintrin.h>
+#if !defined(__IWMMXT__)
 #  include <xmmintrin.h>
-#  define C_FF const m64 mmx_0x00ff = _mm_set1_pi16(0xff)
-#  define C_80 const m64 mmx_0x0080 = _mm_set1_pi16(0x80)
-#  define C_00 const m64 mmx_0x0000 = _mm_setzero_si64()
-#  if defined(Q_OS_WIN)
-#    pragma warning(disable: 4799) // No EMMS at end of function
-#  endif
+#endif
+#define C_FF const m64 mmx_0x00ff = _mm_set1_pi16(0xff)
+#define C_80 const m64 mmx_0x0080 = _mm_set1_pi16(0x80)
+#define C_00 const m64 mmx_0x0000 = _mm_setzero_si64()
+#if defined(Q_OS_WIN)
+#  pragma warning(disable: 4799) // No EMMS at end of function
 #endif
 
 typedef __m64 m64;
