@@ -69,7 +69,7 @@ void qDrawShadeLine(QPainter *p, int x1, int y1, int x2, int y2,
                      int lineWidth, int midLineWidth)
 {
     if (!(p && lineWidth >= 0 && midLineWidth >= 0))        {
-        qWarning("qDrawShadeLine invalid parameters.");
+        qWarning("qDrawShadeLine: Invalid parameters");
         return;
     }
     int tlw = lineWidth*2 + midLineWidth;        // total line width
@@ -188,7 +188,7 @@ void qDrawShadeRect(QPainter *p, int x, int y, int w, int h,
     if (w == 0 || h == 0)
         return;
     if (! (w > 0 && h > 0 && lineWidth >= 0 && midLineWidth >= 0)) {
-        qWarning("qDrawShadeRect(): Invalid parameters");
+        qWarning("qDrawShadeRect: Invalid parameters");
         return;
     }
     QPen oldPen = p->pen();
@@ -291,7 +291,7 @@ void qDrawShadePanel(QPainter *p, int x, int y, int w, int h,
     if (w == 0 || h == 0)
         return;
     if (!(w > 0 && h > 0 && lineWidth >= 0)) {
-        qWarning("qDrawShadePanel() Invalid parameters.");
+        qWarning("qDrawShadePanel: Invalid parameters");
     }
     QColor shade = pal.dark().color();
     QColor light = pal.light().color();
@@ -503,7 +503,7 @@ void qDrawPlainRect(QPainter *p, int x, int y, int w, int h, const QColor &c,
     if (w == 0 || h == 0)
         return;
     if (!(w > 0 && h > 0 && lineWidth >= 0)) {
-        qWarning("qDrawPlainRect() Invalid parameters.");
+        qWarning("qDrawPlainRect: Invalid parameters");
     }
     QPen   oldPen   = p->pen();
     QBrush oldBrush = p->brush();
@@ -839,7 +839,7 @@ void qDrawArrow(QPainter *p, Qt::ArrowType type, Qt::GUIStyle style, bool down,
             break;
 #endif
         default:
-            qWarning("qDrawArrow: Requested GUI style not supported");
+            qWarning("qDrawArrow: Requested unsupported GUI style");
     }
 }
 
