@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -24,22 +24,25 @@
 #ifndef QPAINTER_H
 #define QPAINTER_H
 
-#include "QtCore/qnamespace.h"
-#include "QtCore/qrect.h"
-#include "QtCore/qpoint.h"
-#include "QtGui/qpixmap.h"
-#include "QtGui/qimage.h"
-#include "QtGui/qtextoption.h"
+#include <QtCore/qnamespace.h>
+#include <QtCore/qrect.h>
+#include <QtCore/qpoint.h>
+#include <QtGui/qpixmap.h>
+#include <QtGui/qimage.h>
+#include <QtGui/qtextoption.h>
+#include <QtGui/qdrawutil.h>
 
 #ifndef QT_INCLUDE_COMPAT
-#include "QtGui/qpolygon.h"
-#include "QtGui/qpen.h"
-#include "QtGui/qbrush.h"
-#include "QtGui/qmatrix.h"
-#include "QtGui/qfontinfo.h"
-#include "QtGui/qfontmetrics.h"
+#include <QtGui/qpolygon.h>
+#include <QtGui/qpen.h>
+#include <QtGui/qbrush.h>
+#include <QtGui/qmatrix.h>
+#include <QtGui/qfontinfo.h>
+#include <QtGui/qfontmetrics.h>
 #endif
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 class QBrush;
@@ -56,6 +59,8 @@ class QMatrix;
 class Q_GUI_EXPORT QPainter
 {
     Q_DECLARE_PRIVATE(QPainter)
+    Q_GADGET
+    Q_FLAGS(RenderHint RenderHints)
 
 public:
     enum RenderHint {
@@ -122,6 +127,9 @@ public:
     void setBackground(const QBrush &bg);
     const QBrush &background() const;
 
+    qreal opacity() const;
+    void setOpacity(qreal opacity);
+
     // Clip functions
     QRegion clipRegion() const;
     QPainterPath clipPath() const;
@@ -146,9 +154,17 @@ public:
     const QMatrix &deviceMatrix() const;
     void resetMatrix();
 
+    void setWorldMatrix(const QMatrix &matrix, bool combine = false);
+    const QMatrix &worldMatrix() const;
+
+    QMatrix combinedMatrix() const;
+
     void setMatrixEnabled(bool enabled);
     bool matrixEnabled() const;
 
+    void setWorldMatrixEnabled(bool enabled);
+    bool worldMatrixEnabled() const;
+
     void scale(qreal sx, qreal sy);
     void shear(qreal sh, qreal sv);
     void rotate(qreal a);
@@ -311,6 +327,7 @@ public:
     inline void eraseRect(const QRect &);
 
     void setRenderHint(RenderHint hint, bool on = true);
+    void setRenderHints(RenderHints hints, bool on = true);
     RenderHints renderHints() const;
 
     QPaintEngine *paintEngine() const;
@@ -373,8 +390,6 @@ public:
     static inline QT3_SUPPORT QPaintDevice *redirect(QPaintDevice *pdev)
     { return const_cast<QPaintDevice*>(redirected(pdev)); }
 
-    inline QT3_SUPPORT void setWorldMatrix(const QMatrix &wm, bool combine=false) { setMatrix(wm, combine); }
-    inline QT3_SUPPORT const QMatrix &worldMatrix() const { return matrix(); }
     inline QT3_SUPPORT void setWorldXForm(bool enabled) { setMatrixEnabled(enabled); }
     inline QT3_SUPPORT bool hasWorldXForm() const { return matrixEnabled(); }
     inline QT3_SUPPORT void resetXForm() { resetMatrix(); }
@@ -396,6 +411,7 @@ public:
 #endif
 
 private:
+    Q_DISABLE_COPY(QPainter)
     friend class Q3Painter;
     friend void qt_format_text(const QFont &font, const QRectF &_r, int tf, const QString& str,
                                QRectF *brect, int tabstops, int* tabarray, int tabarraylen,
@@ -417,6 +433,8 @@ private:
     friend class QWin32PaintEnginePrivate;
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(QPainter::RenderHints)
+
 //
 // functions
 //
@@ -798,4 +816,6 @@ inline void QPainter::drawPicture(const QPoint &pt, const QPicture &p)
 }
 #endif
 
+QT_END_HEADER
+
 #endif // QPAINTER_H
