@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -66,7 +66,7 @@ class QPdfEngine : public QPdfBaseEngine
 {
     Q_DECLARE_PRIVATE(QPdfEngine)
 public:
-    QPdfEngine();
+    QPdfEngine(QPrinter::PrinterMode m);
     virtual ~QPdfEngine();
 
     // reimplementations QPaintEngine
@@ -81,19 +81,13 @@ public:
     // end reimplementations QPaintEngine
 
     // reimplementations QPrintEngine
-    void setProperty(PrintEnginePropertyKey key, const QVariant &value);
-    QVariant property(PrintEnginePropertyKey key) const;
-    int metric(QPaintDevice::PaintDeviceMetric) const;
     bool abort() {return false;}
     bool newPage();
-    QPrinter::PrinterState printerState() const {return QPrinter::Idle;}
+    QPrinter::PrinterState printerState() const {return state;}
     // end reimplementations QPrintEngine
 
     void setBrush();
 
-    QRect paperRect() const;
-    QRect pageRect() const;
-
     // ### unused, should have something for this in QPrintEngine
     void setAuthor(const QString &author);
     QString author() const;
@@ -103,39 +97,35 @@ public:
 private:
     Q_DISABLE_COPY(QPdfEngine)
 
-    QPrinter::PageSize pagesize_;
-
-    QIODevice* device_;
-    QFile* outFile_;
+    QPrinter::PrinterState state;
 };
 
 class QPdfEnginePrivate : public QPdfBaseEnginePrivate
 {
     Q_DECLARE_PUBLIC(QPdfEngine)
 public:
-    QPdfEnginePrivate();
+    QPdfEnginePrivate(QPrinter::PrinterMode m);
     ~QPdfEnginePrivate();
 
     void newPage();
-    void setDimensions(int w, int h){width_ = w; height_ = h;}
 
-    QString title, creator, author;
-
-    void setDevice(QIODevice*);
-    void unsetDevice();
-    int width() const {return width_;}
-    int height() const {return height_;}
+    int width() const {
+        QRect r = paperRect();
+        return qRound(r.width()*72./resolution);
+    }
+    int height() const {
+        QRect r = paperRect();
+        return qRound(r.height()*72./resolution);
+    }
 
     void writeHeader();
     void writeTail();
 
-    QPrinter::PageOrder pageOrder;
-    QPrinter::Orientation orientation;
-    bool fullPage;
-
-    int addImage(const QImage &image, bool *bitmap);
+    int addImage(const QImage &image, bool *bitmap, qint64 serial_no);
     int addBrushPattern(const QMatrix &matrix, bool *specifyColor, int *gStateObject);
 
+    QMatrix pageMatrix() const;
+
 private:
     Q_DISABLE_COPY(QPdfEnginePrivate)
 
@@ -149,12 +139,11 @@ private:
     void embedFont(QFontSubset *font);
 
     QVector<int> xrefPositions;
-    int width_, height_;
     QDataStream* stream;
     int streampos;
 
     int writeImage(const QByteArray &data, int width, int height, int depth,
-                   int maskObject, int softMaskObject);
+                   int maskObject, int softMaskObject, bool dct = false);
     void writePage();
 
     int addXrefEntry(int object, bool printostr = true);
@@ -170,6 +159,7 @@ private:
     // various PDF objects
     int pageRoot, catalog, info, graphicsState, patternColorSpace;
     QVector<uint> pages;
+    QHash<qint64, uint> imageCache;
 };
 
 
