@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -27,6 +27,8 @@
 #include <QtGui/qcommonstyle.h>
 #include <QtCore/qpointer.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 #if !defined(QT_NO_STYLE_MOTIF)
@@ -80,6 +82,10 @@ public:
     bool event(QEvent *);
     QPalette standardPalette() const;
 
+protected Q_SLOTS:
+    QIcon standardIconImplementation(StandardPixmap standardIcon, const QStyleOption *opt = 0,
+                                     const QWidget *widget = 0) const;
+
 protected:
     QPointer<QFocusFrame> focus;
     QMotifStyle(QMotifStylePrivate &dd, bool useHighlightCols = false);
@@ -95,4 +101,6 @@ private:
 
 #endif // QT_NO_STYLE_MOTIF
 
+QT_END_HEADER
+
 #endif // QMOTIFSTYLE_H
