@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -29,10 +29,14 @@
 #ifdef QT3_SUPPORT
 #include <QtGui/qaction.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 class Q_GUI_EXPORT QMenuItem : public QAction
 {
+    Q_OBJECT
+
 public:
     QMenuItem();
 
@@ -44,6 +48,9 @@ private:
     void setId(int);
     void setSignalValue(int);
 };
+
+QT_END_HEADER
+
 #endif
 
 #endif // QMENUDATA_H
