@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -34,9 +34,10 @@
 #include "qregexp.h"
 #include "qtimer.h"
 #include "qfileinfo.h"
-#include "q3ptrdict.h" // binary compatibility
+#include "q3ptrdict.h"
 #include "q3cstring.h"
 #include "qcoreapplication.h"
+#include "qftp.h"
 
 #ifndef QT_NO_TEXTCODEC
 #include "qtextcodec.h"
@@ -45,7 +46,6 @@
 //#define Q3FTPPI_DEBUG
 //#define Q3FTPDTP_DEBUG
 
-
 class Q3FtpPI;
 
 class Q3FtpDTP : public QObject
@@ -203,7 +203,7 @@ private:
 
     Q3Socket commandSocket;
     QString replyText;
-    char replyCode[3];
+    signed char replyCode[3];
     State state;
     AbortState abortState;
     QStringList pendingCommands;
@@ -283,16 +283,16 @@ Q3FtpDTP::Q3FtpDTP( Q3FtpPI *p, QObject *parent, const char *name ) :
 {
     clearData();
 
-    connect( &socket, SIGNAL( connected() ),
-	     SLOT( socketConnected() ) );
-    connect( &socket, SIGNAL( readyRead() ),
-	     SLOT( socketReadyRead() ) );
-    connect( &socket, SIGNAL( error(int) ),
-	     SLOT( socketError(int) ) );
-    connect( &socket, SIGNAL( connectionClosed() ),
-	     SLOT( socketConnectionClosed() ) );
-    connect( &socket, SIGNAL( bytesWritten(int) ),
-	     SLOT( socketBytesWritten(int) ) );
+    connect( &socket, SIGNAL(connected()),
+	     SLOT(socketConnected()) );
+    connect( &socket, SIGNAL(readyRead()),
+	     SLOT(socketReadyRead()) );
+    connect( &socket, SIGNAL(error(int)),
+	     SLOT(socketError(int)) );
+    connect( &socket, SIGNAL(connectionClosed()),
+	     SLOT(socketConnectionClosed()) );
+    connect( &socket, SIGNAL(bytesWritten(int)),
+	     SLOT(socketBytesWritten(int)) );
 }
 
 void Q3FtpDTP::setData( QByteArray *ba )
@@ -559,7 +559,8 @@ void Q3FtpDTP::socketReadyRead()
 	    qDebug( "Q3FtpDTP read: %d bytes (total %d bytes)", (int)bytesRead, bytesDone );
 #endif
 	    emit dataTransferProgress( bytesDone, bytesTotal );
-	    data.dev->writeBlock( ba );
+            if (data.dev)       // make sure it wasn't deleted in the slot
+                data.dev->writeBlock( ba );
 	} else {
 #if defined(Q3FTPDTP_DEBUG)
 	    qDebug( "Q3FtpDTP readyRead: %d bytes available (total %d bytes read)", (int)bytesAvailable(), bytesDone );
@@ -659,7 +660,7 @@ bool Q3FtpPI::sendCommands( const QStringList &cmds )
 	return false;
 
     if ( commandSocket.state()!=Q3Socket::Connected || state!=Idle ) {
-	emit error( Q3Ftp::NotConnected, Q3Ftp::tr( "Not connected" ) );
+	emit error( Q3Ftp::NotConnected, QFtp::tr( "Not connected" ) );
 	return true; // there are no pending commands
     }
 
@@ -724,11 +725,11 @@ void Q3FtpPI::error( int e )
     if ( e == Q3Socket::ErrHostNotFound ) {
 	emit connectState( Q3Ftp::Unconnected );
 	emit error( Q3Ftp::HostNotFound,
-		    Q3Ftp::tr( "Host %1 not found" ).arg( commandSocket.peerName() ) );
+		    QFtp::tr( "Host %1 not found" ).arg( commandSocket.peerName() ) );
     } else if ( e == Q3Socket::ErrConnectionRefused ) {
 	emit connectState( Q3Ftp::Unconnected );
 	emit error( Q3Ftp::ConnectionRefused,
-		    Q3Ftp::tr( "Connection refused to host %1" ).arg( commandSocket.peerName() ) );
+		    QFtp::tr( "Connection refused to host %1" ).arg( commandSocket.peerName() ) );
     }
 }
 
@@ -830,7 +831,7 @@ bool Q3FtpPI::processReply()
 		return true;
 	    } else if ( replyCode[0] == 2 ) {
 		state = Idle;
-		emit finished( Q3Ftp::tr( "Connected to host %1" ).arg( commandSocket.peerName() ) );
+		emit finished( QFtp::tr( "Connected to host %1" ).arg( commandSocket.peerName() ) );
 		break;
 	    }
 	    // ### error handling
@@ -847,7 +848,7 @@ bool Q3FtpPI::processReply()
             }
 #else
             state = table[replyCode[0] - 1];
-#endif            
+#endif
 	    break;
 	default:
 	    // ### spontaneous message
@@ -991,7 +992,7 @@ void Q3FtpPI::dtpConnectState( int s )
 	case Q3FtpDTP::CsHostNotFound:
 	case Q3FtpDTP::CsConnectionRefused:
 	    emit error( Q3Ftp::ConnectionRefused,
-			Q3Ftp::tr( "Connection refused for data connection" ) );
+			QFtp::tr( "Connection refused for data connection" ) );
 	    startNextCmd();
 	    return;
 	default:
@@ -1238,23 +1239,23 @@ Q3Ftp::Q3Ftp( QObject *parent, const char *name ) : Q3NetworkProtocol()
 void Q3Ftp::init()
 {
     Q3FtpPrivate *d = ::d( this );
-    d->errorString = tr( "Unknown error" );
+    d->errorString = QFtp::tr( "Unknown error" );
 
     connect( &d->pi, SIGNAL(connectState(int)),
 	    SLOT(piConnectState(int)) );
-    connect( &d->pi, SIGNAL(finished(const QString&)),
-	    SLOT(piFinished(const QString&)) );
-    connect( &d->pi, SIGNAL(error(int,const QString&)),
-	    SLOT(piError(int,const QString&)) );
-    connect( &d->pi, SIGNAL(rawFtpReply(int,const QString&)),
-	    SLOT(piFtpReply(int,const QString&)) );
+    connect( &d->pi, SIGNAL(finished(QString)),
+	    SLOT(piFinished(QString)) );
+    connect( &d->pi, SIGNAL(error(int,QString)),
+	    SLOT(piError(int,QString)) );
+    connect( &d->pi, SIGNAL(rawFtpReply(int,QString)),
+	    SLOT(piFtpReply(int,QString)) );
 
     connect( &d->pi.dtp, SIGNAL(readyRead()),
 	    SIGNAL(readyRead()) );
     connect( &d->pi.dtp, SIGNAL(dataTransferProgress(int,int)),
 	    SIGNAL(dataTransferProgress(int,int)) );
-    connect( &d->pi.dtp, SIGNAL(listInfo(const QUrlInfo&)),
-	    SIGNAL(listInfo(const QUrlInfo&)) );
+    connect( &d->pi.dtp, SIGNAL(listInfo(QUrlInfo)),
+	    SIGNAL(listInfo(QUrlInfo)) );
 }
 
 /*!
@@ -1966,7 +1967,7 @@ void Q3Ftp::startNextCommand()
 	return;
 
     d->error = NoError;
-    d->errorString = tr( "Unknown error" );
+    d->errorString = QFtp::tr( "Unknown error" );
 
     if ( bytesAvailable() )
 	readAll(); // clear the data
@@ -2043,31 +2044,31 @@ void Q3Ftp::piError( int errorCode, const QString &text )
     d->error = (Error)errorCode;
     switch ( currentCommand() ) {
 	case ConnectToHost:
-	    d->errorString = tr( "Connecting to host failed:\n%1" ).arg( text );
+	    d->errorString = QFtp::tr( "Connecting to host failed:\n%1" ).arg( text );
 	    break;
 	case Login:
-	    d->errorString = tr( "Login failed:\n%1" ).arg( text );
+	    d->errorString = QFtp::tr( "Login failed:\n%1" ).arg( text );
 	    break;
 	case List:
-	    d->errorString = tr( "Listing directory failed:\n%1" ).arg( text );
+	    d->errorString = QFtp::tr( "Listing directory failed:\n%1" ).arg( text );
 	    break;
 	case Cd:
-	    d->errorString = tr( "Changing directory failed:\n%1" ).arg( text );
+	    d->errorString = QFtp::tr( "Changing directory failed:\n%1" ).arg( text );
 	    break;
 	case Get:
-	    d->errorString = tr( "Downloading file failed:\n%1" ).arg( text );
+	    d->errorString = QFtp::tr( "Downloading file failed:\n%1" ).arg( text );
 	    break;
 	case Put:
-	    d->errorString = tr( "Uploading file failed:\n%1" ).arg( text );
+	    d->errorString = QFtp::tr( "Uploading file failed:\n%1" ).arg( text );
 	    break;
 	case Remove:
-	    d->errorString = tr( "Removing file failed:\n%1" ).arg( text );
+	    d->errorString = QFtp::tr( "Removing file failed:\n%1" ).arg( text );
 	    break;
 	case Mkdir:
-	    d->errorString = tr( "Creating directory failed:\n%1" ).arg( text );
+	    d->errorString = QFtp::tr( "Creating directory failed:\n%1" ).arg( text );
 	    break;
 	case Rmdir:
-	    d->errorString = tr( "Removing directory failed:\n%1" ).arg( text );
+	    d->errorString = QFtp::tr( "Removing directory failed:\n%1" ).arg( text );
 	    break;
 	default:
 	    d->errorString = text;
@@ -2092,7 +2093,7 @@ void Q3Ftp::piConnectState( int state )
     emit stateChanged( d->state );
     if ( d->close_waitForStateChange ) {
 	d->close_waitForStateChange = false;
-	piFinished( tr( "Connection closed" ) );
+	piFinished( QFtp::tr( "Connection closed" ) );
     }
 }
 
@@ -2186,8 +2187,8 @@ bool Q3Ftp::checkConnection( Q3NetworkOperation *op )
 {
     Q3FtpPrivate *d = ::d( this );
     if ( state() == Unconnected && !d->npWaitForLoginDone ) {
-	connect( this, SIGNAL(listInfo(const QUrlInfo&)),
-		this, SLOT(npListInfo(const QUrlInfo&)) );
+	connect( this, SIGNAL(listInfo(QUrlInfo)),
+		this, SLOT(npListInfo(QUrlInfo)) );
 	connect( this, SIGNAL(done(bool)),
 		this, SLOT(npDone(bool)) );
 	connect( this, SIGNAL(stateChanged(int)),
@@ -2310,8 +2311,8 @@ void Q3Ftp::npDone( bool err )
     d->npWaitForLoginDone = false;
 
     if ( state() == Unconnected ) {
-	disconnect( this, SIGNAL(listInfo(const QUrlInfo&)),
-		    this, SLOT(npListInfo(const QUrlInfo&)) );
+	disconnect( this, SIGNAL(listInfo(QUrlInfo)),
+		    this, SLOT(npListInfo(QUrlInfo)) );
 	disconnect( this, SIGNAL(done(bool)),
 		    this, SLOT(npDone(bool)) );
 	disconnect( this, SIGNAL(stateChanged(int)),
@@ -2331,18 +2332,18 @@ void Q3Ftp::npStateChanged( int state )
 {
     if ( url() ) {
 	if ( state == Connecting )
-	    emit connectionStateChanged( ConHostFound, tr( "Host %1 found" ).arg( url()->host() ) );
+	    emit connectionStateChanged( ConHostFound, QFtp::tr( "Host %1 found" ).arg( url()->host() ) );
 	else if ( state == Connected )
-	    emit connectionStateChanged( ConConnected, tr( "Connected to host %1" ).arg( url()->host() ) );
+	    emit connectionStateChanged( ConConnected, QFtp::tr( "Connected to host %1" ).arg( url()->host() ) );
 	else if ( state == Unconnected )
-	    emit connectionStateChanged( ConClosed, tr( "Connection to %1 closed" ).arg( url()->host() ) );
+	    emit connectionStateChanged( ConClosed, QFtp::tr( "Connection to %1 closed" ).arg( url()->host() ) );
     } else {
 	if ( state == Connecting )
-	    emit connectionStateChanged( ConHostFound, tr( "Host found" ) );
+	    emit connectionStateChanged( ConHostFound, QFtp::tr( "Host found" ) );
 	else if ( state == Connected )
-	    emit connectionStateChanged( ConConnected, tr( "Connected to host" ) );
+	    emit connectionStateChanged( ConConnected, QFtp::tr( "Connected to host" ) );
 	else if ( state == Unconnected )
-	    emit connectionStateChanged( ConClosed, tr( "Connection closed" ) );
+	    emit connectionStateChanged( ConClosed, QFtp::tr( "Connection closed" ) );
     }
 }
 
@@ -2356,7 +2357,6 @@ void Q3Ftp::npReadyRead()
     emit data( readAll(), operationInProgress() );
 }
 
-// ### unused -- delete in Qt 4.0
 /*!  \internal
 */
 void Q3Ftp::hostFound()
