@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtSVG module of the Qt Toolkit.
 **
@@ -88,8 +88,8 @@ QSvgFontStyle::QSvgFontStyle(QSvgFont *font, QSvgTinyDocument *doc)
 {
 }
 
-QSvgFontStyle::QSvgFontStyle(const QFont &font)
-    : m_font(0), m_pointSize(24), m_qfont(font)
+QSvgFontStyle::QSvgFontStyle(const QFont &font, QSvgTinyDocument *doc)
+    : m_font(0), m_pointSize(24), m_doc(doc), m_qfont(font)
 {
 }
 
@@ -165,6 +165,10 @@ QSvgGradientStyle::QSvgGradientStyle(QGradient *grad, bool resolveBounds)
 
 void QSvgGradientStyle::apply(QPainter *p, const QRectF &rect, QSvgNode *)
 {
+    if (!m_link.isEmpty()) {
+        resolveStops();
+    }
+    
     m_oldFill = p->brush();
 
     //resolving stop colors
@@ -179,6 +183,7 @@ void QSvgGradientStyle::apply(QPainter *p, const QRectF &rect, QSvgNode *)
         }
     }
 
+    QBrush brush;
     //we need to resolve boundries
     //the code is funky i'll have to verify it
     //(testcases right now are bugs/resolve_radial.svg
@@ -194,8 +199,7 @@ void QSvgGradientStyle::apply(QPainter *p, const QRectF &rect, QSvgNode *)
             QLinearGradient gradient(xs, ys,
                                      xf, yf);
             gradient.setStops(m_gradient->stops());
-            QBrush b(gradient);
-            p->setBrush(b);
+            brush = QBrush(gradient);
         } else {
             QRadialGradient *grad = (QRadialGradient*)m_gradient;
             qreal cx, cy, r, fx, fy;
@@ -209,16 +213,20 @@ void QSvgGradientStyle::apply(QPainter *p, const QRectF &rect, QSvgNode *)
             r  = rect.width() * grad->radius();
             fx = rect.width() * grad->focalPoint().x();
             fy = rect.width() * grad->focalPoint().y();
+            //qDebug()<<cx << cy << r << fx << fy;
             QRadialGradient gradient(cx, cy,
                                      r, fx, fy);
             gradient.setStops(m_gradient->stops());
-            QBrush b(gradient);
-            p->setBrush(b);
+            brush = QBrush(gradient);
         }
     } else {
-        QBrush b(*m_gradient);
-        p->setBrush(b);
+        brush = QBrush(*m_gradient);
     }
+    
+    if (!m_matrix.isIdentity())
+        brush.setMatrix(m_matrix);
+    
+    p->setBrush(brush);
 }
 
 void QSvgGradientStyle::revert(QPainter *p)
@@ -226,6 +234,12 @@ void QSvgGradientStyle::revert(QPainter *p)
     p->setBrush(m_oldFill);
 }
 
+
+void QSvgGradientStyle::setMatrix(const QMatrix &mat)
+{
+    m_matrix = mat;
+}
+
 void QSvgGradientStyle::addResolve(qreal offset)
 {
     m_resolvePoints.append(offset);
@@ -287,6 +301,11 @@ QSvgStyleProperty::Type QSvgTransformStyle::type() const
     return TRANSFORM;
 }
 
+QSvgStyle::~QSvgStyle()
+{
+}
+
+
 void QSvgStyle::apply(QPainter *p, const QRectF &rect, QSvgNode *node)
 {
     if (quality) {
@@ -328,12 +347,16 @@ void QSvgStyle::apply(QPainter *p, const QRectF &rect, QSvgNode *node)
     //animated transforms have to be applied
     //_after_ the original object transformations
     if (!animateTransforms.isEmpty()) {
-        QList<QSvgAnimateTransform*>::const_iterator itr;
+        QList<QSvgRefCounter<QSvgAnimateTransform> >::const_iterator itr;
         for (itr = animateTransforms.constBegin(); itr != animateTransforms.constEnd();
              ++itr) {
             (*itr)->apply(p, rect, node);
         }
     }
+
+    if (opacity) {
+        opacity->apply(p, rect, node);
+    }
 }
 
 void QSvgStyle::revert(QPainter *p)
@@ -369,7 +392,7 @@ void QSvgStyle::revert(QPainter *p)
     //animated transforms need to be reverted _before_
     //the native transforms
     if (!animateTransforms.isEmpty()) {
-        QList<QSvgAnimateTransform*>::const_iterator itr;
+        QList<QSvgRefCounter<QSvgAnimateTransform> >::const_iterator itr;
         itr = animateTransforms.constBegin();
         //only need to rever the first one because that
         //one has the original world matrix for the primitve
@@ -385,6 +408,10 @@ void QSvgStyle::revert(QPainter *p)
     if (animateColor) {
         animateColor->revert(p);
     }
+
+    if (opacity) {
+        opacity->revert(p);
+    }
 }
 
 QSvgAnimateTransform::QSvgAnimateTransform(int startMs, int endMs, int byMs )
@@ -692,3 +719,48 @@ void QSvgFontStyle::setTextAnchor(const QString &anchor)
 {
     m_textAnchor = anchor;
 }
+
+QSvgOpacityStyle::QSvgOpacityStyle(qreal opacity)
+    : m_opacity(opacity)
+{
+    
+}
+
+void QSvgOpacityStyle::apply(QPainter *p, const QRectF &, QSvgNode *)
+{
+    m_oldOpacity = p->opacity();
+    p->setOpacity(m_opacity);
+}
+
+void QSvgOpacityStyle::revert(QPainter *p)
+{
+    p->setOpacity(m_oldOpacity);
+}
+
+QSvgStyleProperty::Type QSvgOpacityStyle::type() const
+{
+    return OPACITY;
+}
+
+void QSvgGradientStyle::setStopLink(const QString &link, QSvgTinyDocument *doc)
+{
+    m_link = link;
+    m_doc  = doc;
+}
+
+void QSvgGradientStyle::resolveStops()
+{
+    if (!m_link.isEmpty() && m_doc) {
+        QSvgStyleProperty *prop = m_doc->scopeStyle(m_link);
+        if (prop) {
+            if (prop->type() == QSvgStyleProperty::GRADIENT) {
+                QSvgGradientStyle *st =
+                    static_cast<QSvgGradientStyle*>(prop);
+                st->resolveStops();
+                m_gradient->setStops(st->qgradient()->stops());
+            }
+        }
+        m_link = QString();
+    }
+}
+
