@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -24,64 +24,6 @@
 #ifndef TOKEN_H
 #define TOKEN_H
 
-enum PP_Token {
-    PP_NOTOKEN,
-    PP_IDENTIFIER,
-    PP_INTEGER_LITERAL,
-    PP_CHARACTER_LITERAL,
-    PP_STRING_LITERAL,
-    PP_LANGLE,
-    PP_RANGLE,
-    PP_LPAREN,
-    PP_RPAREN,
-    PP_COMMA,
-    PP_FIRST_STATEMENT, PP_DEFINE = PP_FIRST_STATEMENT,
-    PP_UNDEF,
-    PP_IF,
-    PP_IFDEF,
-    PP_IFNDEF,
-    PP_ELIF,
-    PP_ELSE,
-    PP_ENDIF,
-    PP_INCLUDE,
-    PP_HASHHASH,
-    PP_HASH, PP_LAST_STATEMENT = PP_HASH,
-    PP_DEFINED,
-    PP_PLUS,
-    PP_MINUS,
-    PP_STAR,
-    PP_SLASH,
-    PP_PERCENT,
-    PP_HAT,
-    PP_AND,
-    PP_OR,
-    PP_TILDE,
-    PP_NOT,
-    PP_LTLT,
-    PP_GTGT,
-    PP_EQEQ,
-    PP_NE,
-    PP_LE,
-    PP_GE,
-    PP_ANDAND,
-    PP_OROR,
-    PP_QUESTION,
-    PP_COLON,
-    PP_FLOATING_LITERAL,
-    PP_QUOTE,
-    PP_SINGLEQUOTE,
-    PP_DIGIT,
-    PP_CHARACTER,
-    PP_WHITESPACE,
-    PP_NEWLINE,
-    PP_CPP_COMMENT,
-    PP_C_COMMENT,
-    PP_MOC_INCLUDE_END,
-    PP_QT_SIGNALS,
-    PP_QT_SLOTS,
-    PP_INCOMPLETE
-};
-
 enum Token {
     NOTOKEN,
     IDENTIFIER,
@@ -198,6 +140,8 @@ enum Token {
     BREAK,
     CONTINUE,
     GOTO,
+    SIGNALS,
+    SLOTS,
     RETURN,
     Q_META_TOKEN_BEGIN,
     Q_OBJECT_TOKEN = Q_META_TOKEN_BEGIN,
@@ -207,16 +151,22 @@ enum Token {
     Q_FLAGS_TOKEN,
     Q_DECLARE_FLAGS_TOKEN,
     Q_DECLARE_INTERFACE_TOKEN,
+    Q_DECLARE_METATYPE_TOKEN,
     Q_CLASSINFO_TOKEN,
     Q_INTERFACES_TOKEN,
-    SIGNALS,
-    SLOTS,
+    Q_SIGNALS_TOKEN,
+    Q_SLOTS_TOKEN,
     Q_PRIVATE_SLOT_TOKEN,
+    Q_MOC_COMPAT_TOKEN,
+    Q_QT3_SUPPORT_TOKEN,
+    Q_INVOKABLE_TOKEN,
+    Q_SCRIPTABLE_TOKEN,
     Q_META_TOKEN_END,
     SPECIAL_TREATMENT_MARK = Q_META_TOKEN_END,
     MOC_INCLUDE_BEGIN,
     MOC_INCLUDE_END,
-    MOC_NEXT_IS_IDENTIFIER,
+    CPP_COMMENT,
+    C_COMMENT,
     FLOATING_LITERAL,
     HASH,
     QUOTE,
@@ -225,7 +175,72 @@ enum Token {
     CHARACTER,
     NEWLINE,
     WHITESPACE,
-    INCOMPLETE
+    BACKSLASH,
+    INCOMPLETE,
+
+    PP_DEFINE,
+    PP_UNDEF,
+    PP_IF,
+    PP_IFDEF,
+    PP_IFNDEF,
+    PP_ELIF,
+    PP_ELSE,
+    PP_ENDIF,
+    PP_INCLUDE,
+    PP_HASHHASH,
+    PP_HASH,
+    PP_DEFINED,
+    PP_INCOMPLETE,
+
+    PP_MOC_TRUE,
+    PP_MOC_FALSE,
+
+    PP_NOTOKEN = NOTOKEN,
+    PP_IDENTIFIER = IDENTIFIER,
+    PP_INTEGER_LITERAL = INTEGER_LITERAL,
+    PP_CHARACTER_LITERAL = CHARACTER_LITERAL,
+    PP_STRING_LITERAL = STRING_LITERAL,
+    PP_LANGLE = LANGLE,
+    PP_RANGLE = RANGLE,
+    PP_LPAREN = LPAREN,
+    PP_RPAREN = RPAREN,
+    PP_COMMA = COMMA,
+    PP_PLUS = PLUS,
+    PP_MINUS = MINUS,
+    PP_STAR = STAR,
+    PP_SLASH = SLASH,
+    PP_PERCENT = PERCENT,
+    PP_HAT = HAT,
+    PP_AND = AND,
+    PP_OR = OR,
+    PP_TILDE = TILDE,
+    PP_NOT = NOT,
+    PP_LTLT = LTLT,
+    PP_GTGT = GTGT,
+    PP_EQEQ = EQEQ,
+    PP_NE = NE,
+    PP_LE = LE,
+    PP_GE = GE,
+    PP_ANDAND = ANDAND,
+    PP_OROR = OROR,
+    PP_QUESTION = QUESTION,
+    PP_COLON = COLON,
+    PP_FLOATING_LITERAL = FLOATING_LITERAL,
+    PP_QUOTE = QUOTE,
+    PP_SINGLEQUOTE = SINGLEQUOTE,
+    PP_DIGIT = DIGIT,
+    PP_CHARACTER = CHARACTER,
+    PP_WHITESPACE = WHITESPACE,
+    PP_NEWLINE = NEWLINE,
+    PP_CPP_COMMENT = CPP_COMMENT,
+    PP_C_COMMENT = C_COMMENT,
+    PP_BACKSLASH = BACKSLASH
 };
 
+// for debugging only
+#if defined(DEBUG_MOC)
+const char *tokenTypeName(Token t);
+#endif
+
+typedef Token PP_Token;
 #endif // TOKEN_H
