@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -66,9 +66,9 @@ bool RCCFileInfo::writeDataInfo(FILE *out, RCCResourceLibrary::Format format)
 {
     //some info
     if(format == RCCResourceLibrary::C_Code) {
-        if(locale != QLocale::c())
+        if(language != QLocale::C)
             fprintf(out, "  // %s [%d::%d]\n  ", resourceName().toLatin1().constData(),
-                    locale.country(), locale.language());
+                    country, language);
         else
             fprintf(out, "  // %s\n  ", resourceName().toLatin1().constData());
     }
@@ -104,10 +104,10 @@ bool RCCFileInfo::writeDataInfo(FILE *out, RCCResourceLibrary::Format format)
             fprintf(out, ",");
 
         //locale
-        qt_rcc_write_number(out, locale.country(), 2, format);
+        qt_rcc_write_number(out, country, 2, format);
         if(format == RCCResourceLibrary::C_Code)
             fprintf(out, ",");
-        qt_rcc_write_number(out, locale.language(), 2, format);
+        qt_rcc_write_number(out, language, 2, format);
         if(format == RCCResourceLibrary::C_Code)
             fprintf(out, ",");
 
@@ -144,7 +144,7 @@ qint64 RCCFileInfo::writeDataBlob(FILE *out, qint64 offset, RCCResourceLibrary::
         }
     }
 #endif // QT_NO_COMPRESS
-    
+
     //some info
     if(format == RCCResourceLibrary::C_Code)
         fprintf(out, "  // %s\n  ", fileInfo.absoluteFilePath().toLatin1().constData());
@@ -216,7 +216,7 @@ RCCResourceLibrary::~RCCResourceLibrary()
     delete root;
 }
 
-bool RCCResourceLibrary::interpretResourceFile(QIODevice *inputDevice, QString fname, QString currentPath)
+bool RCCResourceLibrary::interpretResourceFile(QIODevice *inputDevice, QString fname, QString currentPath, bool ignoreErrors)
 {
     if (!currentPath.isEmpty() && !currentPath.endsWith(QLatin1String("/")))
         currentPath += '/';
@@ -226,6 +226,8 @@ bool RCCResourceLibrary::interpretResourceFile(QIODevice *inputDevice, QString f
         QString errorMsg;
         int errorLine, errorColumn;
         if(!document.setContent(inputDevice, &errorMsg, &errorLine, &errorColumn)) {
+            if(ignoreErrors)
+                return true;
             fprintf(stderr, "RCC Parse Error:%s:%d:%d [%s]\n", fname.toLatin1().constData(),
                     errorLine, errorColumn, errorMsg.toLatin1().constData());
             return false;
@@ -239,9 +241,20 @@ bool RCCResourceLibrary::interpretResourceFile(QIODevice *inputDevice, QString f
         for (QDomElement child = root.firstChild().toElement(); !child.isNull();
              child = child.nextSibling().toElement()) {
             if (child.tagName() == QLatin1String(TAG_RESOURCE)) {
-                QLocale lang = QLocale::c();
-                if (child.hasAttribute(ATTRIBUTE_LANG))
-                    lang = QLocale(child.attribute(ATTRIBUTE_LANG));
+                QLocale::Language language = QLocale::c().language();
+                QLocale::Country country = QLocale::c().country();
+
+                if (child.hasAttribute(ATTRIBUTE_LANG)) {
+                    QString attribute = child.attribute(ATTRIBUTE_LANG);
+                    QLocale lang = QLocale(attribute);
+                    language = lang.language();
+                    if(2 == attribute.length()) {
+                        // Language only
+                        country = QLocale::AnyCountry;
+                    } else {
+                        country = lang.country();
+                    }
+                }
 
                 QString prefix;
                 if (child.hasAttribute(ATTRIBUTE_PREFIX))
@@ -251,13 +264,12 @@ bool RCCResourceLibrary::interpretResourceFile(QIODevice *inputDevice, QString f
                 if (!prefix.endsWith(QLatin1String("/")))
                     prefix += '/';
 
-
                 for (QDomNode res = child.firstChild(); !res.isNull(); res = res.nextSibling()) {
                     if (res.toElement().tagName() == QLatin1String(TAG_FILE)) {
 
                         QString fileName(res.firstChild().toText().data());
-                        if (fileName.isEmpty())
-                            fprintf(stderr, "Warning: Null node in XML\n");
+                        if (ignoreErrors && fileName.isEmpty())
+                            fprintf(stderr, "RCC: Warning: Null node in XML\n");
 
                         QString alias;
                         if (res.toElement().hasAttribute(ATTRIBUTE_ALIAS))
@@ -279,14 +291,16 @@ bool RCCResourceLibrary::interpretResourceFile(QIODevice *inputDevice, QString f
                         alias = QDir::cleanPath(alias);
                         while (alias.startsWith("../"))
                             alias.remove(0, 3);
-                        alias = prefix + alias;
+                        alias = QDir::cleanPath(mResourceRoot) + prefix + alias;
 
                         QFileInfo file(currentPath + fileName);
                         if (!file.exists()) {
-                            fprintf(stderr, "Cannot find file: %s\n", fileName.toLatin1().constData());
-                            continue ;
+                            if(ignoreErrors)
+                                continue;
+                            fprintf(stderr, "RCC: Error: Cannot find file '%s'\n", fileName.toLatin1().constData());
+                            return false;
                         } else if (file.isFile()) {
-                            addFile(alias, RCCFileInfo(alias.section('/', -1), file, lang,
+                            addFile(alias, RCCFileInfo(alias.section('/', -1), file, language, country,
                                                        RCCFileInfo::NoFlags, compressLevel, compressThreshold));
                         } else {
                             QDir dir;
@@ -305,7 +319,7 @@ bool RCCResourceLibrary::interpretResourceFile(QIODevice *inputDevice, QString f
                                 if(children[i].fileName() != QLatin1String(".") &&
                                    children[i].fileName() != QLatin1String(".."))
                                     addFile(alias + children[i].fileName(),
-                                            RCCFileInfo(children[i].fileName(), children[i], lang,
+                                            RCCFileInfo(children[i].fileName(), children[i], language, country,
                                                         RCCFileInfo::NoFlags, compressLevel, compressThreshold));
                             }
                         }
@@ -314,8 +328,8 @@ bool RCCResourceLibrary::interpretResourceFile(QIODevice *inputDevice, QString f
             }
         }
     }
-    if(this->root == 0) {
-        fprintf(stderr, "No resources in resource description.\n");
+    if(!ignoreErrors && this->root == 0) {
+        fprintf(stderr, "RCC: Warning: No resources in resource description.\n");
         return false;
     }
     return true;
@@ -329,14 +343,14 @@ bool RCCResourceLibrary::addFile(const QString &alias, const RCCFileInfo &file)
         return false;
     }
     if(!root)
-        root = new RCCFileInfo("", QFileInfo(), QLocale(), RCCFileInfo::Directory);
+        root = new RCCFileInfo("", QFileInfo(), QLocale::C, QLocale::AnyCountry, RCCFileInfo::Directory);
 
     RCCFileInfo *parent = root;
     const QStringList nodes = alias.split('/');
     for(int i = 1; i < nodes.size()-1; ++i) {
         const QString node = nodes.at(i);
         if(!parent->children.contains(node)) {
-            RCCFileInfo *s = new RCCFileInfo(node, QFileInfo(), QLocale(), RCCFileInfo::Directory);
+            RCCFileInfo *s = new RCCFileInfo(node, QFileInfo(), QLocale::C, QLocale::AnyCountry, RCCFileInfo::Directory);
             s->parent = parent;
             parent->children.insert(node, s);
             parent = s;
@@ -352,11 +366,11 @@ bool RCCResourceLibrary::addFile(const QString &alias, const RCCFileInfo &file)
     return true;
 }
 
-bool RCCResourceLibrary::readFiles()
+bool RCCResourceLibrary::readFiles(bool ignoreErrors)
 {
     //read in data
     if (mVerbose)
-        fprintf(stderr, "Processing %d files\n", mFileNames.size());
+        fprintf(stderr, "Processing %d files [%d]\n", mFileNames.size(), ignoreErrors);
     for (int i=0; i<mFileNames.size(); ++i) {
         QFile fileIn;
         QString fname = mFileNames.at(i), pwd;
@@ -379,7 +393,7 @@ bool RCCResourceLibrary::readFiles()
         if (mVerbose)
             fprintf(stderr, "Interpreting %s\n", fname.toLatin1().constData());
 
-        if (!interpretResourceFile(&fileIn, fname, pwd))
+        if (!interpretResourceFile(&fileIn, fname, pwd, ignoreErrors))
             return false;
     }
     return true;
