@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -33,6 +33,7 @@
 #include <QtDebug>
 #include <QFile>
 #include <QHash>
+#include <QPair>
 #include <QStringList>
 #include <QDateTime>
 #include <QRegExp>
@@ -49,6 +50,30 @@
         } \
     } while (0)
 
+static QString classNameForObjectName(const QDomElement &widget, const QString &objectName)
+{
+    QList<QDomElement> widgetStack;
+    widgetStack.append(widget);
+    while (!widgetStack.isEmpty()) {
+        QDomElement w = widgetStack.takeFirst();
+        QDomElement child = w.firstChild().toElement();
+        while (!child.isNull()) {
+            if (child.tagName() == QLatin1String("property")
+                && child.attribute(QLatin1String("name")) == QLatin1String("name")) {
+                QDomElement name = child.firstChild().toElement();
+                DomString str;
+                str.read(name);
+                if (str.text() == objectName)
+                    return w.attribute(QLatin1String("class"));
+            } else if (child.tagName() == QLatin1String("widget")) {
+                widgetStack.prepend(child);
+            }
+            child = child.nextSibling().toElement();
+        }
+    }
+    return QString();
+}
+
 DomUI *Ui3Reader::generateUi4(const QDomElement &widget)
 {
     QDomNodeList nl;
@@ -57,18 +82,22 @@ DomUI *Ui3Reader::generateUi4(const QDomElement &widget)
     QString objClass = getClassName(widget);
     if (objClass.isEmpty())
         return 0;
+    QString objName = getObjectName(widget);
 
     DomUI *ui = new DomUI;
     ui->setAttributeVersion(QLatin1String("4.0"));
 
     QString pixmapFunction = QLatin1String("qPixmapFromMimeSource");
     QStringList ui_tabstops;
+    QStringList ui_custom_slots;
     QList<DomInclude*> ui_includes;
     QList<DomWidget*> ui_toolbars;
     QList<DomWidget*> ui_menubars;
     QList<DomAction*> ui_action_list;
     QList<DomActionGroup*> ui_action_group_list;
     QList<DomCustomWidget*> ui_customwidget_list;
+    QList<DomConnection*> ui_connection_list;
+    QList<QPair<int, int> > ui_connection_lineinfo_list;
     QString author, comment, exportMacro;
     QString klass;
 
@@ -94,6 +123,19 @@ DomUI *Ui3Reader::generateUi4(const QDomElement &widget)
             comment = n.firstChild().toText().data();
         } else if (tagName == QLatin1String("exportMacro")) {
             exportMacro = n.firstChild().toText().data();
+        } else if ( n.tagName() == "includehints" ) {
+            QDomElement n2 = n.firstChild().toElement();
+            while ( !n2.isNull() ) {
+                if ( n2.tagName() == "includehint" ) {
+                    QString name = n2.firstChild().toText().data();
+
+                    DomInclude *incl = new DomInclude();
+                    incl->setText(fixHeaderName(name));
+                    incl->setAttributeLocation(n.attribute(QLatin1String("location"), QLatin1String("local")));
+                    ui_includes.append(incl);
+                }
+                n2 = n2.nextSibling().toElement();
+            }
         } else if (tagName == QLatin1String("includes")) {
             QDomElement n2 = n.firstChild().toElement();
             while (!n2.isNull()) {
@@ -257,6 +299,66 @@ DomUI *Ui3Reader::generateUi4(const QDomElement &widget)
                 }
                 n2 = n2.nextSibling().toElement();
             }
+        } else if (tagName == QLatin1String("connections")) {
+            QDomElement n2 = n.firstChild().toElement();
+            while (!n2.isNull()) {
+                if (n2.tagName().toLower() == QLatin1String("connection")) {
+
+                    DomConnection *connection = new DomConnection;
+                    connection->read(n2);
+
+                    QString signal = fixMethod(connection->elementSignal());
+                    QString slot = fixMethod(connection->elementSlot());
+                    connection->setElementSignal(signal);
+                    connection->setElementSlot(slot);
+
+                    ui_connection_list.append(connection);
+                    ui_connection_lineinfo_list.append(
+                        QPair<int, int>(n2.lineNumber(), n2.columnNumber()));
+                }
+                n2 = n2.nextSibling().toElement();
+            }
+        } else if (tagName == QLatin1String("slots")) {
+            QDomElement n2 = n.firstChild().toElement();
+            while (!n2.isNull()) {
+                if (n2.tagName().toLower() == QLatin1String("slot")) {
+                    QString name = n2.firstChild().toText().data();
+                    ui_custom_slots.append(fixMethod(name));
+                }
+                n2 = n2.nextSibling().toElement();
+            }
+        }
+    }
+
+    // validate the connections
+    for (int i = 0; i < ui_connection_list.size(); ++i) {
+        DomConnection *conn = ui_connection_list.at(i);
+        QPair<int, int> lineinfo = ui_connection_lineinfo_list.at(i);
+        QString sender = conn->elementSender();
+        QString senderClass = fixClassName(classNameForObjectName(widget, sender));
+        QString signal = conn->elementSignal();
+        QString receiver = conn->elementReceiver();
+        QString receiverClass = fixClassName(classNameForObjectName(widget, receiver));
+        QString slot = conn->elementSlot();
+
+        if (!WidgetInfo::isValidSignal(senderClass, signal)) {
+            errorInvalidSignal(signal, sender, senderClass,
+                               lineinfo.first, lineinfo.second);
+        } else if (!WidgetInfo::isValidSlot(receiverClass, slot)) {
+            bool resolved = false;
+            if (objName == receiver) {
+                // see if it's a custom slot
+                foreach (QString cs, ui_custom_slots) {
+                    if (cs == slot) {
+                        resolved = true;
+                        break;
+                    }
+                }
+            }
+            if (!resolved) {
+                errorInvalidSlot(slot, receiver, receiverClass,
+                                 lineinfo.first, lineinfo.second);
+            }
         }
     }
 
@@ -341,6 +443,12 @@ DomUI *Ui3Reader::generateUi4(const QDomElement &widget)
         ui->setElementIncludes(includes);
     }
 
+    if (ui_connection_list.size()) {
+        DomConnections *connections = new DomConnections();
+        connections->setElementConnection(ui_connection_list);
+        ui->setElementConnections(connections);
+    }
+
     ui->setAttributeStdSetDef(stdsetdef);
 
     return ui;
@@ -350,7 +458,6 @@ QString Ui3Reader::fixActionProperties(QList<DomProperty*> &properties,
                                        bool isActionGroup)
 {
     QString objectName;
-    bool hasMenuText = false;
 
     QMutableListIterator<DomProperty*> it(properties);
     while (it.hasNext()) {
@@ -362,11 +469,10 @@ QString Ui3Reader::fixActionProperties(QList<DomProperty*> &properties,
         } else if (isActionGroup && name == QLatin1String("exclusive")) {
             // continue
         } else if (isActionGroup) {
-            errorInvalidProperty(name, objectName, isActionGroup ? QLatin1String("QActionGroup") : QLatin1String("QAction"));
+            errorInvalidProperty(name, objectName, isActionGroup ? QLatin1String("QActionGroup") : QLatin1String("QAction"), -1, -1);
             delete prop;
             it.remove();
         } else if (name == QLatin1String("menuText")) {
-            hasMenuText = true;
             prop->setAttributeName("text");
         } else if (name == QLatin1String("text")) {
             prop->setAttributeName("iconText");
@@ -379,7 +485,7 @@ QString Ui3Reader::fixActionProperties(QList<DomProperty*> &properties,
         } else if (name == QLatin1String("on")) {
             prop->setAttributeName(QLatin1String("checked"));
         } else if (!WidgetInfo::isValidProperty(QLatin1String("QAction"), name)) {
-            errorInvalidProperty(name, objectName, isActionGroup ? QLatin1String("QActionGroup") : QLatin1String("QAction"));
+            errorInvalidProperty(name, objectName, isActionGroup ? QLatin1String("QActionGroup") : QLatin1String("QAction"), -1, -1);
             delete prop;
             it.remove();
         }
@@ -454,8 +560,6 @@ DomWidget *Ui3Reader::createWidget(const QDomElement &w, const QString &widgetCl
     QList<DomActionRef*> ui_action_list;
     QList<DomWidget*> ui_mainwindow_child_list;
 
-    bool needPolish = false;
-
     createProperties(w, &ui_property_list, className);
     createAttributes(w, &ui_attribute_list, className);
 
@@ -470,13 +574,36 @@ DomWidget *Ui3Reader::createWidget(const QDomElement &w, const QString &widgetCl
 
     QDomElement e = w.firstChild().toElement();
     bool inQ3ToolBar = className == QLatin1String("Q3ToolBar");
+    bool inQ3GroupBox = (className == QLatin1String("Q3GroupBox")) || (className == QLatin1String("Q3ButtonGroup"));
 
     while (!e.isNull()) {
         QString t = e.tagName().toLower();
         if (t == QLatin1String("vbox") || t == QLatin1String("hbox") || t == QLatin1String("grid")) {
             DomLayout *lay = createLayout(e);
             Q_ASSERT(lay != 0);
+
             if (ui_layout_list.isEmpty()) {
+                DomProperty *pmargin = 0;
+
+                if (inQ3GroupBox) {
+                    foreach (DomProperty *prop, lay->elementProperty()) {
+                        if (prop->attributeName() == QLatin1String("margin")) {
+                            pmargin = prop;
+                            break;
+                        }
+                    }
+
+                    if (! pmargin) {
+                        pmargin = new DomProperty();
+                        pmargin->setAttributeName(QLatin1String("margin"));
+                        pmargin->setElementNumber(0);
+
+                        QList<DomProperty*> plist = lay->elementProperty();
+                        plist.append(pmargin);
+                        lay->setElementProperty(plist);
+                    }
+                }
+
                 ui_layout_list.append(lay);
             } else {
                 // it's not possible to have more than one layout for widget!
@@ -560,12 +687,6 @@ DomWidget *Ui3Reader::createWidget(const QDomElement &w, const QString &widgetCl
             ui_item_list.append(item);
         }
 
-        QString s = getClassName(e);
-        if (s == QLatin1String("QDataTable") || s == QLatin1String("QDataBrowser")) {
-            if (isFrameworkCodeGenerated(e))
-                 needPolish = true;
-        }
-
         e = e.nextSibling().toElement();
     }
 
@@ -613,7 +734,6 @@ DomLayout *Ui3Reader::createLayout(const QDomElement &w)
     createProperties(w, &ui_property_list, className);
     createAttributes(w, &ui_attribute_list, className);
 
-    bool needPolish = false;
     QDomElement e = w.firstChild().toElement();
     while (!e.isNull()) {
         QString t = e.tagName().toLower();
@@ -627,13 +747,6 @@ DomLayout *Ui3Reader::createLayout(const QDomElement &w)
             ui_item_list.append(lay_item);
         }
 
-        QString s = getClassName(e);
-        if (s == QLatin1String("QDataTable")
-                 || s == QLatin1String("QDataBrowser")) {
-            if (isFrameworkCodeGenerated(e))
-                 needPolish = true;
-        }
-
         e = e.nextSibling().toElement();
     }
 
@@ -748,60 +861,96 @@ void Ui3Reader::fixLayoutMargin(DomLayout *ui_layout)
     }
 }
 
+void Ui3Reader::findDerivedFontProperties(const QDomElement &n, DomFont &result) const
+{
+    bool italic = false;
+    bool bold = false;
+    bool underline = false;
+    bool strikeout = false;
+
+    QDomNode pn = n.parentNode();
+    while(!pn.isNull()) {
+        for (QDomElement e=pn.firstChild().toElement(); !e.isNull(); e = e.nextSibling().toElement()) {
+            if (e.tagName().toLower() == QLatin1String("property") &&
+                e.attribute(QLatin1String("name")) == QLatin1String("font")) {
+                QDomElement f = e.firstChild().toElement();
+                for (QDomElement fp = f.firstChild().toElement(); !fp.isNull(); fp = fp.nextSibling().toElement()) {
+                    QString name = fp.tagName().toLower();
+                    QString text = fp.text();
+                    if (!italic && name == QLatin1String("italic")) {
+                        italic = true;
+                        if (text == QLatin1String("true") || text == QLatin1String("1"))
+                            result.setElementItalic(true);
+                    } else if (!bold && name == QLatin1String("bold")) {
+                        bold = true;
+                        if (text == QLatin1String("true") || text == QLatin1String("1"))
+                            result.setElementBold(true);
+                    } else if (!underline && name == QLatin1String("underline")) {
+                        underline = true;
+                        if (text == QLatin1String("true") || text == QLatin1String("1"))
+                            result.setElementUnderline(true);
+                    } else if (!strikeout && name == QLatin1String("strikeout")) {
+                        strikeout = true;
+                        if (text == QLatin1String("true") || text == QLatin1String("1"))
+                            result.setElementStrikeOut(true);
+                    } else if (name == QLatin1String("family")) {
+                        if (result.elementFamily().isEmpty())
+                            result.setElementFamily(text.toAscii());
+                    } else if (name == QLatin1String("pointsize")) {
+                        if (!result.elementPointSize())
+                            result.setElementPointSize(text.toInt());
+                    }
+                }
+            }
+        }
+        pn = pn.parentNode();
+    }
+}
+
 void Ui3Reader::createProperties(const QDomElement &n, QList<DomProperty*> *properties,
                                  const QString &className)
 {
     QString objectName;
 
+    bool wordWrapFound = false;
+
     for (QDomElement e=n.firstChild().toElement(); !e.isNull(); e = e.nextSibling().toElement()) {
         if (e.tagName().toLower() == QLatin1String("property")) {
             QString name = e.attribute(QLatin1String("name"));
 
             // changes in QPalette
             if (name == QLatin1String("colorGroup")
-                    || name == QLatin1String("paletteForegroundColor")
-                    || name == QLatin1String("paletteBackgroundColor")
-                    || name == QLatin1String("backgroundMode")
-                    || name == QLatin1String("backgroundOrigin")
-                    || name == QLatin1String("paletteBackgroundPixmap")
-                    || name == QLatin1String("backgroundBrush")) {
-                errorInvalidProperty(name, objectName, className);
+                || name == QLatin1String("paletteForegroundColor")
+                || name == QLatin1String("paletteBackgroundColor")
+                || name == QLatin1String("backgroundMode")
+                || name == QLatin1String("backgroundOrigin")
+                || name == QLatin1String("paletteBackgroundPixmap")
+                || name == QLatin1String("backgroundBrush")) {
+                errorInvalidProperty(name, objectName, className, n.lineNumber(), n.columnNumber());
                 continue;
             }
 
             // changes in QFrame
             if (name == QLatin1String("contentsRect")) {
-                errorInvalidProperty(name, objectName, className);
+                errorInvalidProperty(name, objectName, className, n.lineNumber(), n.columnNumber());
                 continue;
             }
 
             // changes in QWidget
             if (name == QLatin1String("underMouse")
-                    || name == QLatin1String("ownFont")) {
-                errorInvalidProperty(name, objectName, className);
+                || name == QLatin1String("ownFont")) {
+                errorInvalidProperty(name, objectName, className, n.lineNumber(), n.columnNumber());
                 continue;
             }
 
             if (name == QLatin1String("font")) {
-                // For the boolean properties (italic, bold, underline, strikeout),
-                // Ui 3 files store true as "1", but DomFont::read() expects "true",
-                // so we have to convert them
                 QDomElement f = e.firstChild().toElement();
-                for (QDomElement fp = f.firstChild().toElement(); !fp.isNull(); fp = fp.nextSibling().toElement()) {
-                    QString fpTag = fp.tagName().toLower();
-                    if (fpTag == QLatin1String("italic") ||
-                        fpTag == QLatin1String("bold") ||
-                        fpTag == QLatin1String("underline") ||
-                        fpTag == QLatin1String("strikeout")) {
-                        QDomText text = fp.firstChild().toText();
-                        if (!text.isNull()) {
-                            if (text.data() == QLatin1String("1"))
-                                text.setData(QLatin1String("true"));
-                            else if (text.data() == QLatin1String("0"))
-                                text.setData(QLatin1String("false"));
-                        }
-                    }
-                }
+                DomFont font;
+                findDerivedFontProperties(f, font);
+                e.removeChild(f);
+                QDomDocument doc = e.ownerDocument();
+                f = font.write(doc);
+                e.appendChild(f);
             }
 
             DomProperty *prop = readProperty(e);
@@ -817,8 +966,14 @@ void Ui3Reader::createProperties(const QDomElement &n, QList<DomProperty*> *prop
                 }
             }
 
+            // objectName
+            if (name == QLatin1String("name")) {
+                objectName = prop->elementCstring();
+                continue;
+            }
+
             if (className == QLatin1String("Line")
-                    && prop->attributeName() == QLatin1String("orientation")) {
+                && prop->attributeName() == QLatin1String("orientation")) {
                 delete prop;
                 continue;
             }
@@ -870,6 +1025,14 @@ void Ui3Reader::createProperties(const QDomElement &n, QList<DomProperty*> *prop
 
             name = prop->attributeName(); // sync the name
 
+            if (className == QLatin1String("QLabel") && name == QLatin1String("alignment")) {
+                QString v = prop->elementSet();
+
+                if (v.contains(QRegExp("\\bWordBreak\\b")))
+                    wordWrapFound = true;
+            }
+
+
             // resolve the flags and enumerator
             if (prop->kind() == DomProperty::Set) {
                 QStringList flags = prop->elementSet().split(QLatin1Char('|'));
@@ -910,13 +1073,14 @@ void Ui3Reader::createProperties(const QDomElement &n, QList<DomProperty*> *prop
                 prop->setElementEnum(e);
             }
 
+
             if (className.size()
-                    && !(className == QLatin1String("QLabel") && name == QLatin1String("buddy"))
-                    && !(name == QLatin1String("buttonGroupId"))
-                    && !(name == QLatin1String("frameworkCode"))
-                    && !(name == QLatin1String("database"))) {
+                && !(className == QLatin1String("QLabel") && name == QLatin1String("buddy"))
+                && !(name == QLatin1String("buttonGroupId"))
+                && !(name == QLatin1String("frameworkCode"))
+                && !(name == QLatin1String("database"))) {
                 if (!WidgetInfo::isValidProperty(className, name)) {
-                    errorInvalidProperty(name, objectName, className);
+                    errorInvalidProperty(name, objectName, className, n.lineNumber(), n.columnNumber());
                     delete prop;
                 } else {
                     properties->append(prop);
@@ -926,6 +1090,15 @@ void Ui3Reader::createProperties(const QDomElement &n, QList<DomProperty*> *prop
             }
         }
     }
+    if (className == QLatin1String("QLabel")) {
+        DomProperty *wordWrap = new DomProperty();
+        wordWrap->setAttributeName(QLatin1String("wordWrap"));
+        if (wordWrapFound)
+            wordWrap->setElementBool(QLatin1String("true"));
+        else
+            wordWrap->setElementBool(QLatin1String("false"));
+        properties->append(wordWrap);
+    }
 }
 
 DomProperty *Ui3Reader::readProperty(const QDomElement &e)
@@ -938,7 +1111,15 @@ DomProperty *Ui3Reader::readProperty(const QDomElement &e)
     DomProperty *p = new DomProperty;
     p->read(e);
 
-    if (p->kind() == DomProperty::Unknown) {
+    if (p->kind() == DomProperty::Number) {
+        QString value = e.firstChild().toElement().firstChild().nodeValue();
+
+        if (value.contains(QLatin1Char('.'))) {
+            p->setElementDouble(value.toDouble());
+        }
+    }
+
+    else if (p->kind() == DomProperty::Unknown) {
         delete p;
         p = 0;
     }
@@ -1003,3 +1184,24 @@ QString Ui3Reader::fixType(const QString &t) const
     }
     return newText;
 }
+
+QString Ui3Reader::fixMethod(const QString &method) const
+{
+    const QByteArray normalized = QMetaObject::normalizedSignature(method.toLatin1());
+    QByteArray result;
+    int index = normalized.indexOf('(');
+    if (index == -1)
+        return QLatin1String(normalized);
+    result.append(normalized.left(++index));
+    int limit = normalized.length()-1;
+    while (index < limit) {
+        QByteArray type;
+        while ((index < limit) && (normalized.at(index) != ','))
+            type.append(normalized.at(index++));
+        result.append(fixType(QLatin1String(type)).toLatin1());
+        if ((index < limit) && (normalized.at(index) == ','))
+            result.append(normalized.at(index++));
+    }
+    result.append(normalized.mid(index));
+    return QLatin1String(result);
+}
