@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -21,6 +21,10 @@
 **
 ****************************************************************************/
 
+/*
+TRANSLATOR qdesigner_internal::ListWidgetEditor
+*/
+
 #include "listwidgeteditor.h"
 #include <findicondialog_p.h>
 #include <iconloader_p.h>
@@ -141,8 +145,6 @@ void ListWidgetEditor::on_moveItemDownButton_clicked()
 void ListWidgetEditor::on_listWidget_currentRowChanged(int)
 {
     updateEditor();
-    ui.itemTextLineEdit->selectAll();
-    ui.itemTextLineEdit->setFocus();
 }
 
 void ListWidgetEditor::on_listWidget_itemChanged(QListWidgetItem *)
@@ -183,9 +185,7 @@ void ListWidgetEditor::on_previewPixmapItemButton_clicked()
     QString qrc_path;
 
     QIcon icon = item->icon();
-    if (icon.isNull()) {
-        file_path = m_form->absoluteDir().absolutePath();
-    } else {
+    if (!icon.isNull()) {
         file_path = m_form->core()->iconCache()->iconToFilePath(icon);
         qrc_path = m_form->core()->iconCache()->iconToQrcPath(icon);
     }
@@ -251,7 +251,9 @@ void ListWidgetEditor::updateEditor()
         itemText = item->text();
         itemIcon = item->icon();
     }
+    int cursorPos = ui.itemTextLineEdit->cursorPosition();
     ui.itemTextLineEdit->setText(itemText);
+    ui.itemTextLineEdit->setCursorPosition(cursorPos);
     ui.previewPixmapItemButton->setIcon(itemIcon);
     ui.deletePixmapItemButton->setEnabled(!itemIcon.isNull());
 }
