@@ -58,6 +58,7 @@ QDesignerMenu::QDesignerMenu(QWidget *parent)
 
     setContextMenuPolicy(Qt::DefaultContextMenu);
     setAcceptDrops(true); // ### fake
+    setSeparatorsCollapsible(false);
 
     m_adjustSizeTimer = new QTimer(this);
     connect(m_adjustSizeTimer, SIGNAL(timeout()), this, SLOT(slotAdjustSizeNow()));
@@ -504,7 +505,7 @@ void QDesignerMenu::paintEvent(QPaintEvent *event)
     }
 
     QRect g = actionGeometry(current);
-    drawSelection(&p, g.adjusted(1, 1, -3, -2));
+    drawSelection(&p, g.adjusted(1, 1, -3, -3));
 }
 
 bool QDesignerMenu::dragging() const
@@ -763,11 +764,14 @@ void QDesignerMenu::moveLeft()
 {
     if (parentMenu()) {
         hide();
-    } else if (/*QDesignerMenuBar *mb = */parentMenuBar()) {
-#if 0 //. ### disabled for now.. it's a bit confusing
-        hide();
-        mb->moveLeft();
-#endif
+    } else {
+        closeMenuChain();
+        if (QDesignerMenuBar *mb = parentMenuBar()) {
+            if (QApplication::layoutDirection() == Qt::LeftToRight)
+                mb->moveLeft();
+            else
+                mb->moveRight();
+        }
     }
     updateCurrentAction();
 }
@@ -779,7 +783,10 @@ void QDesignerMenu::moveRight()
     if (qobject_cast<SpecialMenuAction*>(action) || action->isSeparator()) {
         closeMenuChain();
         if (QDesignerMenuBar *mb = parentMenuBar()) {
-            mb->moveRight();
+            if (QApplication::layoutDirection() == Qt::LeftToRight)
+                mb->moveRight();
+            else
+                mb->moveLeft();
         }
     } else {
         m_lastSubMenuIndex = -1; // force a refresh
@@ -1222,7 +1229,7 @@ void QDesignerMenu::drawSelection(QPainter *p, const QRect &r)
     p->save();
 
     QColor c = Qt::blue;
-    p->setPen(c);
+    p->setPen(QPen(c, 1));
     c.setAlpha(32);
     p->setBrush(c);
     p->drawRect(r);
