@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -145,6 +145,8 @@ public:
     QString relativePath(const QString &path) const
         { return m_resource_file.relativePath(path); }
 
+    QString lastResourceOpenDirectory() const;
+
     virtual bool reload();
     virtual bool save();
     QString errorMessage() const { return m_resource_file.errorMessage(); }
@@ -160,6 +162,7 @@ signals:
 private:
     ResourceFile m_resource_file;
     bool m_dirty;
+    QString m_lastResourceDir;
 };
 
 } // namespace qdesigner_internal
