@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -37,7 +37,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#ifdef Q_OS_WIN32
+#ifdef Q_OS_WIN
 #include <windows.h> // for Sleep
 #endif
 #ifdef Q_OS_UNIX
@@ -406,14 +406,14 @@
 
 /*! \fn void QTest::keyEvent(KeyAction action, QWidget *widget, Qt::Key key, Qt::KeyboardModifiers modifier = Qt::NoModifier, int delay=-1)
 
-    Sends a Qt key event to \a widget with the given \a key and an associated \a action. Optionally, a keyboard \a modifier can be specified, as well as a \a delay (in milliseconds) of the test after sending the event.
+    Sends a Qt key event to \a widget with the given \a key and an associated \a action. Optionally, a keyboard \a modifier can be specified, as well as a \a delay (in milliseconds) of the test before sending the event.
 */
 
 /*! \fn void QTest::keyEvent(KeyAction action, QWidget *widget, char ascii, Qt::KeyboardModifiers modifier = Qt::NoModifier, int delay=-1)
 
     \overload
 
-    Sends a Qt key event to \a widget with the given key \a ascii and an associated \a action. Optionally, a keyboard \a modifier can be specified, as well as a \a delay (in milliseconds) of the test after sending the event.
+    Sends a Qt key event to \a widget with the given key \a ascii and an associated \a action. Optionally, a keyboard \a modifier can be specified, as well as a \a delay (in milliseconds) of the test before sending the event.
 
 */
 
@@ -458,7 +458,7 @@
 
     Simulates clicking a \a sequence of keys on a \a
     widget. Optionally, a keyboard \a modifier can be specified as
-    well as a \a delay (in milliseconds) of the test after each key
+    well as a \a delay (in milliseconds) of the test before each key
     click.
 
     Example:
@@ -478,7 +478,7 @@
     Simulates pressing a mouse \a button with an optional \a modifier
     on a \a widget.  The position is defined by \a pos; the default
     position is the center of the widget. If \a delay is specified,
-    the test will wait for the specified amount of milliseconds after
+    the test will wait for the specified amount of milliseconds before
     the press.
 
     \sa QTest::mouseRelease(), QTest::mouseClick()
@@ -490,7 +490,7 @@
     on a \a widget.  The position of the release is defined by \a pos;
     the default position is the center of the widget. If \a delay is
     specified, the test will wait for the specified amount of
-    milliseconds after releasing the button.
+    milliseconds before releasing the button.
 
     \sa QTest::mousePress(), QTest::mouseClick()
 */
@@ -501,7 +501,7 @@
     on a \a widget.  The position of the click is defined by \a pos;
     the default position is the center of the widget. If \a delay is
     specified, the test will wait for the specified amount of
-    milliseconds after pressing and after releasing the button.
+    milliseconds before pressing and before releasing the button.
 
     \sa QTest::mousePress(), QTest::mouseRelease()
 */
@@ -512,7 +512,7 @@
     modifier on a \a widget.  The position of the click is defined by
     \a pos; the default position is the center of the widget. If \a
     delay is specified, the test will wait for the specified amount of
-    milliseconds after each press and release.
+    milliseconds before each press and release.
 
     \sa QTest::mouseClick()
 */
@@ -521,7 +521,7 @@
 
     Moves the mouse pointer to a \a widget. If \a pos is not
     specified, the mouse pointer moves to the center of the widget. If
-    a \a delay (in milliseconds) is given, the test will wait after
+    a \a delay (in milliseconds) is given, the test will wait before
     moving the mouse pointer.
 */
 
@@ -561,6 +561,91 @@
     \sa QCOMPARE()
 */
 
+/*!
+    \fn char *QTest::toString(const QLatin1String &string)
+    \overload
+
+    Returns a textual representation of the given \a string.
+*/
+
+/*!
+    \fn char *QTest::toString(const QString &string)
+    \overload
+
+    Returns a textual representation of the given \a string.
+*/
+
+/*!
+    \fn char *QTest::toString(const QTime &time)
+    \overload
+
+    Returns a textual representation of the given \a time.
+*/
+
+/*!
+    \fn char *QTest::toString(const QDate &date)
+    \overload
+
+    Returns a textual representation of the given \a date.
+*/
+
+/*!
+    \fn char *QTest::toString(const QDateTime &dateTime)
+    \overload
+
+    Returns a textual representation of the date and time specified by
+    \a dateTime.
+*/
+
+/*!
+    \fn char *QTest::toString(const QChar &character)
+    \overload
+
+    Returns a textual representation of the given \a character.
+*/
+
+/*!
+    \fn char *QTest::toString(const QPoint &point)
+    \overload
+
+    Returns a textual representation of the given \a point.
+*/
+
+/*!
+    \fn char *QTest::toString(const QSize &size)
+    \overload
+
+    Returns a textual representation of the given \a size.
+*/
+
+/*!
+    \fn char *QTest::toString(const QRect &rectangle)
+    \overload
+
+    Returns a textual representation of the given \a rectangle.
+*/
+
+/*!
+    \fn char *QTest::toString(const QPointF &point)
+    \overload
+
+    Returns a textual representation of the given \a point.
+*/
+
+/*!
+    \fn char *QTest::toString(const QSizeF &size)
+    \overload
+
+    Returns a textual representation of the given \a size.
+*/
+
+/*!
+    \fn char *QTest::toString(const QRectF &rectangle)
+    \overload
+
+    Returns a textual representation of the given \a rectangle.
+*/
+
 /*! \fn void QTest::qWait(int ms)
 
     Waits for \a ms milliseconds. While waiting, events will be processed and
@@ -706,6 +791,7 @@ static void qParseArgs(int argc, char *argv[])
          " -xml       : Outputs results as XML document\n"
          " -lightxml  : Outputs results as stream of XML tags\n"
          " -o filename: Writes all output into a file\n"
+         " -silent    : Only outputs warnings and failures\n"
          " -v1        : Print enter messages for each testfunction\n"
          " -v2        : Also print out each QVERIFY/QCOMPARE/QTEST\n"
          " -vs        : Print every signal emitted\n"
@@ -731,6 +817,8 @@ static void qParseArgs(int argc, char *argv[])
             QTestLog::setLogMode(QTestLog::XML);
         } else if (strcmp(argv[i], "-lightxml") == 0) {
             QTestLog::setLogMode(QTestLog::LightXML);
+        } else if (strcmp(argv[i], "-silent") == 0) {
+            QTestLog::setVerboseLevel(-1);
         } else if (strcmp(argv[i], "-v1") == 0) {
             QTestLog::setVerboseLevel(1);
         } else if (strcmp(argv[i], "-v2") == 0) {
@@ -808,6 +896,18 @@ static void qParseArgs(int argc, char *argv[])
     }
 }
 
+struct QTestDataSetter
+{
+    QTestDataSetter(QTestData *data)
+    {
+        QTestResult::setCurrentTestData(data);
+    }
+    ~QTestDataSetter()
+    {
+        QTestResult::setCurrentTestData(0);
+    }
+};
+
 /*!
     Call init(), slot_data(), slot(), slot(), slot()..., cleanup()
     If data is set then it is the only test that is performed
@@ -849,8 +949,8 @@ static bool qInvokeTestMethod(const char *slotName, const char *data=0)
             do {
                 if (!data || !qstrcmp(data, table.testData(curDataIndex)->dataTag())) {
                     foundFunction = true;
-                    if (!table.isEmpty())
-                        QTestResult::setCurrentTestData(table.testData(curDataIndex));
+                    QTestDataSetter s(table.isEmpty() ? static_cast<QTestData *>(0) 
+                                                      : table.testData(curDataIndex));
                     QTestResult::setCurrentTestLocation(QTestResult::InitFunc);
                     QMetaObject::invokeMethod(QTest::currentTestObject, "init");
                     if (QTest::skipCurrentTest)
@@ -866,7 +966,6 @@ static bool qInvokeTestMethod(const char *slotName, const char *data=0)
                     QTestResult::setCurrentTestLocation(QTestResult::CleanupFunc);
                     QMetaObject::invokeMethod(QTest::currentTestObject, "cleanup");
                     QTestResult::setCurrentTestLocation(QTestResult::NoWhere);
-                    QTestResult::setCurrentTestData(0);
 
                     if (QTest::skipCurrentTest)
                         // check whether SkipAll was requested
@@ -893,6 +992,7 @@ static bool qInvokeTestMethod(const char *slotName, const char *data=0)
 
     QTest::skipCurrentTest = false;
     QTestResult::finishedCurrentTestFunction();
+    QTestResult::setCurrentTestData(0);
     delete[] slot;
 
     return true;
@@ -906,7 +1006,7 @@ void *fetchData(QTestData *data, const char *tagName, int typeId)
 
     int idx = data->parent()->indexOf(tagName);
 
-    if (idx == -1) {
+    if (idx == -1 || idx >= data->dataCount()) {
         qFatal("QFETCH: Requested testdata '%s' not available, check your _data function.",
                 tagName);
     }
@@ -923,14 +1023,32 @@ void *fetchData(QTestData *data, const char *tagName, int typeId)
 } // namespace
 
 /*!
-    Executes tests declared in \a testObject. Optionally, the command line arguments
-    \a argc and \a argv can be provided. For a list of recognized arguments, read
-    \l {QTestLib Command Line Arguments}.
+    Executes tests declared in \a testObject. In addition, the private slots
+    \c{initTestCase()}, \c{cleanupTestCase()}, \c{init()} and \c{cleanup()}
+    are executed if they exist. See \l{Creating a test} for more details.
+
+    Optionally, the command line arguments \a argc and \a argv can be provided.
+    For a list of recognized arguments, read \l {QTestLib Command Line Arguments}.
 
     For stand-alone tests, the convenience macro \l QTEST_MAIN() can
     be used to declare a main method that parses the command line arguments
     and executes the tests.
 
+    The following example will run all tests in \c MyFirstTestObject and
+    \c{MySecondTestObject}:
+
+    \code
+    MyFirstTestObject test1;
+    QTest::qExec(&test1);
+
+    MySecondTestObject test2;
+    QTest::qExec(&test2);
+    \endcode
+
+    Note: This function is not reentrant, only one test can run at a time. A
+    test that was executed with qExec() can't run another test via qExec() and
+    threads are not allowed to call qExec() simultaneously.
+
     \sa QTEST_MAIN()
 */
 int QTest::qExec(QObject *testObject, int argc, char **argv)
@@ -939,10 +1057,12 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
     try {
 #endif
 
-#if defined(Q_CC_MSVC)
+#if defined(Q_OS_WIN)
     SetErrorMode(SetErrorMode(0) | SEM_NOGPFAULTERRORBOX);
 #endif
 
+    QTestResult::reset();
+
     QTEST_ASSERT(testObject);
     QTEST_ASSERT(!currentTestObject);
     currentTestObject = testObject;
@@ -1246,7 +1366,7 @@ void QTest::qSleep(int ms)
 {
     QTEST_ASSERT(ms > 0);
 
-#ifdef Q_OS_WIN32
+#ifdef Q_OS_WIN
     Sleep(uint(ms));
 #else
     struct timespec ts = { ms / 1000, (ms % 1000) * 1000 * 1000 };
@@ -1410,6 +1530,26 @@ bool QTest::compare_string_helper(const char *t1, const char *t2, const char *ac
     \internal
 */
 
+/*! \fn bool QTest::qCompare(QString const &t1, QLatin1String const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
+/*! \fn bool QTest::qCompare(QLatin1String const &t1, QString const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
+/*! \fn bool QTest::qCompare(QStringList const &t1, QStringList const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
+/*! \fn bool QTest::qCompare(QFlags<T> const &t1, T const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
+/*! \fn bool QTest::qCompare(QFlags<T> const &t1, int const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
 /*! \fn bool QTest::qTest(const T& actual, const char *elementName, const char *actualStr, const char *expected, const char *file, int line)
     \internal
 */
@@ -1426,5 +1566,6 @@ bool QTest::compare_string_helper(const char *t1, const char *t2, const char *ac
     \internal
 */
 
-
-
+/*! \fn int QTest::qt_snprintf(char *str, int size, const char *format, ...)
+    \internal
+*/
